/**
    @file appdata.h

    Application specific data needed all around the application.

    Copyright (c) 2004-2006 Nokia Corporation. All rights reserved.
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
    02111-1307 USA
*/


#ifndef _APPDATA_H
#define _APPDATA_H

/*Hildon*/
#include <hildon-widgets/hildon-app.h>
#include <hildon-widgets/hildon-appview.h>
#include <hildon-widgets/hildon-find-toolbar.h>
#include <hildon-widgets/hildon-file-system-model.h>

/*OSSO*/
#include <libosso.h>

/*GTK*/
#include <gtk/gtk.h>

/*GConf*/
#include <gconf/gconf-client.h>

#include "feed.h"

#include "cache.h"

#define RSS_NEWS_READER_HELP_TOPIC "Applications_RssReader_mainview"
#define RSS_NEWS_READER_HELP_ADD_FEED "Applications_RssReader_addafeeddialog"
#define RSS_NEWS_READER_HELP_REFRESH "Applications_RssReader_refreshdialog"
#define RSS_NEWS_READER_HELP_MANAGE "Applications_RssReader_managefoldersdialog"
#define RSS_NEWS_READER_HELP_DETAILS "Applications_RssReader_feeddetailsdialog"
#define RSS_NEWS_READER_HELP_SETTINGS "Applications_RssReader_settingsdialog"
#define RSS_NEWS_READER_HELP_FEED_DIRECTORY "Applications_RssReader_feeddirectorydialog"
#define RSS_NEWS_READER_HELP_IMPORT_OPML_FILE "Applications_RssReader_importOPMLdialog"

/* osso iap actions */
#define OSSO_IAP_NO_ACTION 1
#define OSSO_IAP_POPUP_REFRESH_DIALOG 2 /* used to be OSSO_IAP_NORMAL */
#define OSSO_IAP_REFRESH_FEED_DIRECTORY 3
/*Why this? same as refresh_all_no_dialog
#define OSSO_IAP_REFRESH_NO_DIALOG 4
*/
#define OSSO_IAP_REFRESH_ALL_NO_DIALOG 5
#define OSSO_IAP_REFRESH_FEED 6
#define OSSO_IAP_NEW_SUBSCRIPTION 7
#define OSSO_IAP_LAUNCH_WEB_BROWSER 8
#define OSSO_IAP_DOWNLOAD_IMAGES_FOR_DISPLAY 9

enum {
    SFM_NORMAL,
    SFM_INTERRUPTED,
    SFM_SEARCH,
    SFM_DOWNLOAD,
    SFM_DOWNLOAD_IMAGES /*tvh: added this */
};

/*
  Application UI data
*/
typedef struct _AppUIData AppUIData;
struct _AppUIData {
    /* Used to control whether or not the statistics should be shown */
    gboolean feed_displayed;
    gboolean start_on_background;
    gboolean errors_in_refreshing;
    gboolean feed_directory_cancelled;
    gboolean network_disconnected;
    gboolean connected;
    gboolean application_topped;
    //gboolean download_cancelled;
    gboolean download_stopped;
    
    /* LOW MEMORY state */
    gboolean low_mem ;
    /* When a first level dialog is closed by pressing esc, the key-release-event must not be handled,
       otherwise the application will return to normal state if being in full screen state while the
       dialog was open. This is where this flag is needed */
    gboolean dialog_closed;
    
    /* Automatic updates */
    guint automatic_update_source;
    
    /* Source for handling download results */
    guint download_dequeue_source;
    
    /* Find on page */
    gchar *find_string;
    
    /* IAP related stuff */
    int iap_action;
    feedPtr new_subscription;
    gint new_subscription_flags;
    gchar *source;
    const gchar *filter;
    
    int vertical_scroll_window_position;
    
    /* File chooser model*/
    HildonFileSystemModel *model;

    /* Drag and drop */
    nodePtr draggednode;
    
    /* Feed directory */
    gboolean feed_directory_loading;
    //gboolean receiving_feeds;
    
    
    /* View items */
    HildonApp *app;
    HildonAppView *main_view;

    GtkMenu *main_menu_bar;
    GtkWidget *html_scrollpane; 
    GtkWidget *search_label;
    guint rss_search_found ; /* Number of search items found */
    GtkWidget *dialog;
    GtkWidget *refresh_dialog;

    /* Add feed dialog */
    GtkWidget *add_feed_dialog;
    /*tvh:12.Jan.06*/
    GtkWidget *importopmldialog;

    struct fp_prop_ui_data *ui_data;
    GtkWidget *newfeedbtn;
    GtkWidget *changefolderbtn;
    GtkWidget *cancelbtn;
    
    /* Progress / Search field */
    GtkWidget *var_field;
    GtkWidget *search_field;
    GtkWidget *progress_bar;
    int search_string_count;
    int search_mode;
    int progressbar_mode;
    gint rss_current_download;
    gint rss_max_download;
    /* Number of requests pending in all three queues*/
    guint pending_requests_nr;
    /* Number of requests downloaded but not yet processed,
    and pending in the results Async Queue */
    guint downloaded_requests_nr;

    /*gtkhtml specific for loading images */
    gint images_current_download;
    gint images_max_download;
    gboolean downloading_images;
    /* hidden used for counting number of images */
    /*GtkWidget * hidden_htmlwidget; */
    //GtkWidget * htmlwidget;  /* TODO: needed?*/
    GSList *img_req_list;   
    //struct request * img_req;
    //gboolean writing_page ;
    // if the page has finished loading
    gboolean html_displayed;

    /* Toolbar */
    GtkWidget *main_toolbar;
    GtkToolButton *refresh_button;
    GtkToolButton *search_button;
    GtkToolButton *unsubscribe_button;
    GtkToolButton *feed_properties_button;
    GtkToolButton *new_feed_button;
    GtkToolButton *manage_folders_button;
    GtkWidget *button_image_stop;
    GtkWidget *button_image_search;
    GtkWidget *button_image_refresh;    
    GtkToggleToolButton *folders_list_toggle_button;

    /* Find toolbar */
    HildonFindToolbar *find_toolbar;
    
    /* Access to menu items */
    GtkCheckMenuItem *menuitem_folders;
    GtkCheckMenuItem *menuitem_fullscreen;
    GtkCheckMenuItem *menuitem_normal;
    GtkCheckMenuItem *menuitem_full;

    GtkMenuItem *menuitem_open;
    GtkMenuItem *menuitem_unsubscribe;
    /*tvh: 11.Jan.06*/
    GtkMenuItem *menuitem_importopml;
    GtkMenuItem *menuitem_properties;
    GtkMenuItem *menuitem_cut;
    GtkMenuItem *menuitem_copy;
    GtkMenuItem *menuitem_paste;
    GtkMenuItem *menuitem_selectall;    
    GtkMenuItem *menuitem_refresh_feeds;
    GtkMenuItem *menuitem_feed_directory;
    GtkMenuItem *menuitem_search_results;
    GtkMenuItem *menuitem_find_on_page;
    GtkMenuItem *menuitem_add;
    GtkMenuItem *menuitem_manage_folders;

    /* Interaction with applet */
    nodePtr scroll_to_feed;
    guint32 scroll_to_item;
    
    /*GtkPaned gutter position */
    gint gutter_position;

    /* image cache */
    FileCache *img_cache;
};

typedef struct _AppData AppData;
struct _AppData {
    AppUIData *app_ui_data;
    osso_context_t *osso;
    GConfClient *gconf_client;
};

#endif
