/**
 * @file conf.h Liferea configuration (gconf access and feedlist import)
 *
 * Copyright (C) 2003, 2004 Lars Lindner <lars.lindner@gmx.net>
 * Copyright (C) 2004 Nathan J. Conrad <t98502@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _CONF_H
#define _CONF_H

#include <glib.h>
#include "folder.h"

#define RSS_SETTINGS "/apps/osso/rss-feed-reader"
#define RSS_SETTINGS_REUSE_BROWSER_WINDOW "/apps/osso/rss-feed-reader/reuse-browser-window"
#define RSS_SETTINGS_NEWEST_POSTS_FIRST "/apps/osso/rss-feed-reader/item-sort-order"
#define RSS_SETTINGS_AUTOMATIC_UPDATES "/apps/osso/rss-feed-reader/automatic-update"
#define RSS_SETTINGS_AUTOMATIC_UPDATES_INTERVAL "/apps/osso/rss-feed-reader/update-interval"

/* browsing settings */
#define BROWSE_INSIDE_APPLICATION    "/apps/osso/rss-feed-reader/browse-inside-application"
#define BROWSE_KEY_SETTING        "/apps/osso/rss-feed-reader/browse-key-setting"
#define BROWSER_ID            "/apps/osso/rss-feed-reader/browser_id"
#define BROWSER_PLACE            "/apps/osso/rss-feed-reader/browser_place"
#define BROWSER_COMMAND            "/apps/osso/rss-feed-reader/browser"
#define BROWSER_MODULE            "/apps/osso/rss-feed-reader/browser-module"
#define GNOME_DEFAULT_BROWSER_COMMAND    "gnome-open '%s'"
#define DEFAULT_BROWSER_COMMAND        "mozilla '%s'"
#define DEFAULT_FONT            "/desktop/gnome/interface/font_name"
#define USER_FONT            "/apps/osso/rss-feed-reader/browser-font"

/* item list settings */
#define TIME_FORMAT            "/apps/osso/rss-feed-reader/timeformat"
#define TIME_FORMAT_MODE        "/apps/osso/rss-feed-reader/timeformatmode"

/* feed handling settings */
#define STARTUP_FEED_ACTION        "/apps/osso/rss-feed-reader/startup_feed_action"
#define UPDATE_THREAD_CONCURRENCY    "/apps/osso/rss-feed-reader/update-thread-concurrency"
#define KEEP_FEEDS_IN_MEMORY        "/apps/osso/rss-feed-reader/keep-feeds-in-memory"
#define DISABLE_SUBSCRIPTION_PIPE    "/apps/osso/rss-feed-reader/disable-subscription-pipe"

/* GUI settings and persistency values */
#define SHOW_TRAY_ICON            "/apps/osso/rss-feed-reader/trayicon"
#define SHOW_POPUP_WINDOWS        "/apps/osso/rss-feed-reader/show-popup-windows"
#define DISABLE_MENUBAR            "/apps/osso/rss-feed-reader/disable-menubar"
#define DISABLE_TOOLBAR            "/apps/osso/rss-feed-reader/disable-toolbar"
#define DISABLE_TOOLBAR_FULLSCREEN      "/apps/osso/rss-feed-reader/disable-toolbar-fullscreen"
#define ENABLE_FIND_ON_PAGE             "/apps/osso/rss-feed-reader/enable-find-on-page" 
#define LAST_WINDOW_X            "/apps/osso/rss-feed-reader/last-window-x"
#define LAST_WINDOW_Y            "/apps/osso/rss-feed-reader/last-window-y"
#define LAST_WINDOW_WIDTH        "/apps/osso/rss-feed-reader/last-window-width"
#define LAST_WINDOW_HEIGHT        "/apps/osso/rss-feed-reader/last-window-height"
#define LAST_WINDOW_MAXIMIZED        "/apps/osso/rss-feed-reader/last-window-maximized"
#define LAST_VPANE_POS            "/apps/osso/rss-feed-reader/last-vpane-pos"
#define LAST_HPANE_POS            "/apps/osso/rss-feed-reader/last-hpane-pos"
#define LAST_ITEMLIST_MODE        "/apps/osso/rss-feed-reader/last-itemlist-mode"
#define LAST_ZOOMLEVEL            "/apps/osso/rss-feed-reader/last-zoomlevel"
#define START_FIRST_TIME                "/apps/osso/rss-feed-reader/start-first-time"

/* networking settings */
#define USE_PROXY            "/system/http_proxy/use_http_proxy"
#define PROXY_HOST            "/system/http_proxy/host"
#define PROXY_PORT            "/system/http_proxy/port"
#define PROXY_USEAUTH            "/system/http_proxy/use_authentication"
#define PROXY_USER            "/system/http_proxy/authentication_user"
#define PROXY_PASSWD            "/system/http_proxy/authentication_password"

/** Initializes the configuration 
  */
void conf_init(void);

/** Sets values to certain gconf keys
  */
void conf_load(void);

/* feed/directory list entry manipulation methods */

/** Creates a new cache id
  *
  * @return the new id
  */
gchar *conf_new_id();

/** Loads the feedlist from a file
  */
void conf_load_subscriptions();

/** Schedules a feedlist save into a file
  */
void conf_feedlist_schedule_save();

/** Saves the feedlist into a file
  */
void conf_feedlist_save();

/* methods to modify folder contents */

/** Checks if namespace is enabled in configuration
  * @param nsname namespace
  * @return TRUE if namespace is enabled in configuration
  */
gboolean getNameSpaceStatus(gchar * nsname);

/** Used to enable/disable a namespace in configuration 
  *
  * @param nsname namespace
  * @param enable used to enable or disable a namespace
  */
void setNameSpaceStatus(gchar * nsname, gboolean enable);

/** Checks ia a given conf value is initialized
  *
  * @param valuename the value to check
  * @return TRUE if initialized, FALSE otherwise
  */
gboolean checkConfValueExist(gchar * valuename);

/** Gets a boolean value from gconf
  * 
  * @param valuename the name of the value
  * @return the wanted value
  */
gboolean getBooleanConfValue(gchar * valuename);

/** Gets a string value from gconf
  * 
  * @param valuename the name of the value
  * @return the wanted value
  */
gchar *getStringConfValue(gchar * valuename);

/** Gets a numeric value from gconf
  * 
  * @param valuename the name of the value
  * @return the wanted value
  */
gint getNumericConfValue(gchar * valuename);

/** Sets a boolean value to gconf
  * 
  * @param valuename the name of the value
  * @param value the value to set
  */
void setBooleanConfValue(gchar * valuename, gboolean value);

/** Sets a string value to gconf
  * 
  * @param valuename the name of the value
  * @param value the value to set
  */
void setStringConfValue(gchar * valuename, gchar * value);

/** Sets a numeric value to gconf
  * 
  * @param valuename the name of the value
  * @param value the value to set
  */
void setNumericConfValue(gchar * valuename, gint value);

#endif
