/**
    @file dbus.c

    Copyright (c) 2004-2006 Nokia Corporation. All rights reserved.
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
    02111-1307 USA
*/


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "dbus.h"
#include "conf.h"
#include "debug.h"

#include "callbacks.h"
#include "ui_feedlist.h"
#include "update.h"
#include "support.h"
#include "main.h"
#include "callbacks.h"
#include "ui_feed_directory.h"
#include "gtkhtml/gtkhtml_plugin.h"

extern AppData *app_data;
extern time_t rss_updated;
extern feedPtr searchFeed;
static GTimer *messageTimer = NULL;

/************************************************************************/
/*                        PRIVATE FUNCTIONS                             */
/************************************************************************/

/** Show no memory dialog */

static void ui_show_lowmem_banner(void)
{
    gtk_infoprint( GTK_WINDOW(app_data->app_ui_data->app), 
                    dgettext("ke-recv", "memr_ib_operation_disabled") );
    return;                    
    /*
  GtkWidget * note =  hildon_note_new_information(GTK_WINDOW(app_data->app_ui_data->app),
        dgettext("ke-recv", "memr_ib_operation_disabled") );
  //hildon_note_set_button_text(note, __("sfil_ni_not_enough_memory_ok"));

  gtk_dialog_run(GTK_DIALOG(note));
  gtk_widget_destroy(GTK_WIDGET(note));

  //hildon_appview_set_title(app_data->app_ui_data->app, "");
  //plugin_settings_set_string(SETTINGS_CHESS_SAVE_FILE,"");
    */
}


static
gboolean refresh_stop_from_applet(gpointer data)
{
    handle_download_stopped_by_user();
//    ui_update();
    return FALSE;//destroy the timeout
}
/** Handle the OSSO_IAP_CONNECTED signal
  *
  * @param app_ui_data Application specific data
  */
static void handle_iap_connected (struct iap_event_t *event, AppUIData *app_ui_data)
{
    ULOG_INFO("OSSO_IAP_CONNECTED");
    ULOG_DEBUG("iap action: %d",app_ui_data->iap_action);
    
    if (NULL != app_ui_data) {
    /* Activate the g_cond so that threads can be unblocked */
    download_set_online(TRUE);
    //app_ui_data->connected = TRUE;

    if (app_ui_data->iap_action == OSSO_IAP_POPUP_REFRESH_DIALOG &&
        !app_ui_data->start_on_background)
    {
        ULOG_DEBUG("OSSO_IAP_POPUP_REFRESH_DIALOG");
        /* reset it */
        app_ui_data->iap_action = OSSO_IAP_NO_ACTION;
        on_refresh_action(app_ui_data);
    }
        /* Lets refresh all the feeds without popping up the refresh dialog.
       The applet uses this */
       /* same as REFRESH_ALL_NO_DIALOG */
       /*
    //else if (app_ui_data->iap_action == OSSO_IAP_REFRESH_NO_DIALOG)
    {
        app_ui_data->iap_action = OSSO_IAP_NO_ACTION;
        
        if (SFM_NORMAL == app_ui_data->search_mode) {
            ULOG_DEBUG("Refresh without dialog");

        ui_feedlist_do_for_all(NULL, ACTION_FILTER_FEED,
                    on_refreshbtn_clicked_cb);
        }
    }
    */
    /* If no feed has been selected all feeds are refreshed */
    /* Or applet might ask the same */
    else if (app_ui_data->iap_action == OSSO_IAP_REFRESH_ALL_NO_DIALOG)
    {
        app_ui_data->iap_action = OSSO_IAP_NO_ACTION;
        /*Only refresh if it's not searching or already refreshing. 
        Downloading images is ok */
        if ((SFM_NORMAL == app_ui_data->search_mode) ||
            ((SFM_DOWNLOAD == app_ui_data->search_mode) &&
                (get_counter (RSS_MAX) == 0) ) ) {
            ULOG_DEBUG("Refresh without dialog");
            
            ui_feedlist_do_for_all(NULL, ACTION_FILTER_FEED,
                    on_refreshbtn_clicked_cb);

        }
    }
    
    /* Refresh a single feed */
    else if (app_ui_data->iap_action == OSSO_IAP_REFRESH_FEED)
    {
         ULOG_DEBUG("OSSO_IAP_REFRESH_FEED");
        app_ui_data->iap_action = OSSO_IAP_NO_ACTION;
   
         feed_schedule_update(app_data->app_ui_data->new_subscription,
             app_data->app_ui_data->new_subscription_flags | FEED_REQ_PRIORITY_HIGH |
             FEED_REQ_DOWNLOAD_FAVICON | FEED_REQ_AUTH_DIALOG );

         app_ui_data->new_subscription = NULL;
         app_ui_data->new_subscription_flags = 0;
    }
    /* Refresh the feed directory contents */
    else if (app_ui_data->iap_action == OSSO_IAP_REFRESH_FEED_DIRECTORY)
    {
        //app_ui_data->iap_action = OSSO_IAP_NORMAL;
        ULOG_DEBUG("Refresh feed directory contents");
        app_ui_data->iap_action = OSSO_IAP_NO_ACTION;
        ui_feed_directory_get();
    }  
        else if (app_ui_data->iap_action == OSSO_IAP_NEW_SUBSCRIPTION)
    {
        feedPtr fp = NULL;
        gchar *tmp = NULL;
        
        app_ui_data->iap_action = OSSO_IAP_NO_ACTION;
        ULOG_DEBUG("OSSO_IAP_NEW_SUBSCRIPTION");
                            
            /* Only take action is the add feed dialog is open */
            if(app_ui_data->add_feed_dialog != NULL) {
            fp = feed_new();
            tmp = conf_new_id();
            feed_set_id(fp, tmp);
            g_free(tmp);
        
            feed_set_source(fp, app_ui_data->source);
            feed_set_title(fp, app_ui_data->source);
            feed_set_filter(fp, app_ui_data->filter);
            feed_set_added(fp, time(NULL));
                feed_set_new_subscription(fp, TRUE);
        
                //ui_search_clear_search_results();
    
            if(displayed_node == (nodePtr)searchFeed)
            { ULOG_DEBUG("dbus: handle iap connected");
              ui_mainwindow_show_statistics();
            }
            feed_schedule_update(fp,
                 app_data->app_ui_data->new_subscription_flags | FEED_REQ_PRIORITY_HIGH |
                 FEED_REQ_DOWNLOAD_FAVICON | FEED_REQ_AUTH_DIALOG );

                g_free(app_ui_data->source);
            g_free((gchar *)app_ui_data->filter);

            app_ui_data->source = NULL;
            app_ui_data->filter = NULL;
            app_ui_data->new_subscription_flags = 0;
        
            /*        
            gtk_widget_set_sensitive(app_data->app_ui_data->newfeedbtn, FALSE);
            gtk_widget_set_sensitive(app_data->app_ui_data->changefolderbtn, FALSE);
            */
//              gtk_widget_set_sensitive(app_data->app_ui_data->cancelbtn, FALSE);
            }
    }
        else if (app_ui_data->iap_action == OSSO_IAP_LAUNCH_WEB_BROWSER)
        {
            app_ui_data->iap_action = OSSO_IAP_NO_ACTION;
        ULOG_DEBUG("OSSO_IAP_LAUNCH_WEB_BROWSER");

            if(app_ui_data->source != NULL) {
                open_url(app_ui_data->source);
                g_free(app_ui_data->source);
            }
        }
    /*tvh: Added this handler */
    else if (app_ui_data->iap_action == OSSO_IAP_DOWNLOAD_IMAGES_FOR_DISPLAY)
    {
        /* note: which action to set to? */
        ULOG_DEBUG("handle_iap_connect: OSSO_IAP_DOWNLOAD_IMAGES_FOR_DISPLAY");
        app_ui_data->iap_action = OSSO_IAP_NO_ACTION;
        //gtkhtml_handle_net_connect(app_ui_data->img_req_list);
        gtkhtml_handle_net_connect();           
    }
    /* Take no refreshing action */
    else if (app_ui_data->iap_action == OSSO_IAP_NO_ACTION)
    {
        ULOG_DEBUG("IAP no action");
    }
    }
}

/** Handle the OSSO_IAP_ERROR signal
  *
  * @param app_ui_data Application specific data
  */
static void handle_iap_error (struct iap_event_t *event, AppUIData *app_ui_data)
{

    ULOG_DEBUG("OSSO_IAP_ERROR %d", event->u.error_code);
    //fake the connection for use in sbox
    //handle_iap_connected (event, app_ui_data);
    //return;
    app_ui_data->connected = FALSE;
    
    /* A new subscription was made without an existing connection
       and the user hit cancel in the connection dialog */    
    if (app_ui_data->iap_action == OSSO_IAP_NEW_SUBSCRIPTION)    
    {
        const gchar *source = NULL;
    const gchar *filter = NULL;
    gint flags = 0;
    folderPtr selected_parent = NULL;

    ULOG_DEBUG("Adding a feed without a connection");        
    //app_ui_data->iap_action = OSSO_IAP_NORMAL;                 
    /* reset it */
    app_ui_data->iap_action = OSSO_IAP_NO_ACTION;
    
    source = app_ui_data->source;
    filter = app_ui_data->filter;
     flags = app_ui_data->new_subscription_flags;
    selected_parent = app_ui_data->ui_data->parent_folder;

        /* Search must be cleared before adding new stuff to the tree */    
        //ui_search_clear_search_results();
    
    if(displayed_node == (nodePtr)searchFeed)
    {
        ULOG_DEBUG("dbus: handle_iap_error");
        ui_mainwindow_show_statistics();
    }
    ULOG_DEBUG("Adding: %s", source);
    ui_feedlist_new_subscription_for_fd(source, filter, flags, selected_parent, FALSE, TRUE);
    
    g_free(app_ui_data->source);
    g_free((gchar *)app_ui_data->filter);

    app_ui_data->source = NULL;
    app_ui_data->filter = NULL;
    app_ui_data->new_subscription_flags = 0;
    
    newdialog_destroy();
    }
    else if (app_ui_data->iap_action == OSSO_IAP_LAUNCH_WEB_BROWSER)
    {
        app_ui_data->iap_action = OSSO_IAP_NO_ACTION;
    ULOG_DEBUG("OSSO_IAP_LAUNCH_WEB_BROWSER");

        if(app_ui_data->source != NULL) {
            g_free(app_ui_data->source);
        }
    }
    /*tvh: Added this handler. Should free the img_req_list.
        We haven't pushed the list to download_queue so no worries
        this should be enough
     */
    else if (app_ui_data->iap_action == OSSO_IAP_DOWNLOAD_IMAGES_FOR_DISPLAY)
    {
        /* not sure which action to set to */
        ULOG_DEBUG("handle_iap_error: OSSO_IAP_DOWNLOAD_IMAGES_FOR_DISPLAY");
        app_ui_data->iap_action = OSSO_IAP_NO_ACTION;
        //gtkhtml_handle_net_error(app_data->app_ui_data->img_req_list);
        gtkhtml_handle_net_error();
    }
    
    if (NULL != app_ui_data) {
        if (SFM_SEARCH == app_ui_data->search_mode)
        gtk_infoprint(GTK_WINDOW(app_data->app_ui_data->app), _("rss_ib_cannot_start_refresh"));
    else if(SFM_DOWNLOAD == app_ui_data->search_mode)
    {
        download_cancel_all();

//         on_refresh_finished(FALSE, FALSE);
    
        gtk_infoprint(GTK_WINDOW(app_data->app_ui_data->app),
                  _("rss_ni_error_refreshing_all_feeds"));
    }
    } else {
        ULOG_ERR("Received IAP ERROR without IAP request");
    }
    //2006June18*/
    rss_dbus_refresh_completed();
    //app_ui_data->iap_action = OSSO_IAP_NORMAL;                
    /* reset it */
    app_ui_data->iap_action = OSSO_IAP_NO_ACTION;
    //2006June18: Quit if app started on background. This happens 
    //when refreshing from applet but connection returned error   
    if (app_ui_data->start_on_background )
        on_close(NULL,NULL,NULL);
    
}

/** Handle the OSSO_IAP_DISCONNECTED signal
  *
  * @param app_ui_data Application specific data
  */
static void handle_iap_disconnected (struct iap_event_t *event, AppUIData *app_ui_data)
{
    ULOG_DEBUG("OSSO_IAP_DISCONNECTED");

    //g_assert(app_ui_data != NULL);
    
    /* Always use global app_ui_data here */
    if (app_ui_data->search_mode == SFM_DOWNLOAD) {
        app_ui_data->search_mode = SFM_INTERRUPTED;
    download_cancel_all();

//     on_refresh_finished(FALSE, FALSE);

    gtk_infoprint(GTK_WINDOW(app_data->app_ui_data->app), _("rss_ni_connection_was_lost"));
    app_ui_data->network_disconnected = TRUE;
    download_set_online(FALSE);
    app_ui_data->connected = FALSE;
    }
        //2006June18*/
    rss_dbus_refresh_completed();
    //app_ui_data->iap_action = OSSO_IAP_NORMAL;                
    /* reset it */
    app_ui_data->iap_action = OSSO_IAP_NO_ACTION;
    //2006June18: Quit if app started on background. This happens 
    //when refreshing from applet but connection gave error
    if (app_ui_data->start_on_background )
        on_close(NULL,NULL,NULL);

    /*rss_dbus_refresh_completed();*/
    //app_ui_data->iap_action = OSSO_IAP_NORMAL; ??
}

/************************************************************************/
/*                        PUBLIC FUNCTIONS                              */
/************************************************************************/

/* Define topping */
osso_application_top_cb_f *osso_top_callback(const gchar * arguments,
                         AppData * app_data)
{
    ULOG_DEBUG("********************osso_top_callback********************************");
    //g_assert(app_data);
    //g_assert(app_data->app_ui_data != NULL);
    
    gtk_widget_show(GTK_WIDGET(app_data->app_ui_data->app));
    gtk_window_present(GTK_WINDOW(app_data->app_ui_data->app));
    return NULL;
}

void abort_quit();

gint dbus_req_handler(const gchar * interface,
              const gchar * method,
              GArray * arguments,
              gpointer data, osso_rpc_t * retval)
{
    (void) interface;
    return dbus_message_handler(method, arguments, data, retval);
}

gint dbus_message_handler(const gchar * method,
              GArray * arguments,
              gpointer data, osso_rpc_t * retval)
{
    AppData *app_data = NULL;
    app_data = (AppData *) data;
    AppUIData *app_ui_data;
    gchar *filename = NULL;
    gchar *feed = NULL;
    guint32 nr = 0;

    //g_assert(app_data != NULL);
    //g_assert(app_data->app_ui_data != NULL);
        
    app_ui_data = app_data->app_ui_data;
    ULOG_DEBUG("%s received call for method %s", __FUNCTION__, method);

    (void) arguments;
    //g_assert(method);
    g_return_val_if_fail ((method), OSSO_OK);
    /* Catch the message and define what you want to do with it */
    if (g_ascii_strcasecmp(method, "top_application") == 0) {
        printf("==================================\n");
        printf("=           TOP APPLICATION      =\n");
        printf("==================================\n");
        app_ui_data->start_on_background = FALSE;

        gdk_threads_enter();
        abort_quit();
        gtk_widget_show(GTK_WIDGET(app_data->app_ui_data->app));
        gtk_window_present(GTK_WINDOW(app_data->app_ui_data->app));
        gdk_threads_leave();

        retval->type = DBUS_TYPE_INVALID;
        osso_rpc_free_val (retval);
        return OSSO_OK;
    } else
    if (g_ascii_strcasecmp(method, OSSO_RSS_FEED_READER_REFRESH_SIGNAL) ==
    0) {
    
        //Try to avoid several messages in a short time
      if (g_timer_elapsed(messageTimer, NULL) > 0.2) {
       filename =
           g_strdup_printf("%s" G_DIR_SEPARATOR_S ".osso_rss_feed_reader"
                G_DIR_SEPARATOR_S "background",
                g_get_home_dir());
    
        if (g_file_test(filename, G_FILE_TEST_EXISTS)) {
           g_unlink(filename);
       }
       g_free(filename);    

       gboolean timed = g_array_index(arguments, osso_rpc_t, 0).value.b;
       
       if(timed && !app_ui_data->start_on_background)
       {
           g_timer_reset(messageTimer);
           retval->type = DBUS_TYPE_INVALID;
           osso_rpc_free_val (retval);

           return OSSO_OK;
       }
       
       if (app_ui_data->search_mode != SFM_NORMAL) {
           if ((app_ui_data->search_mode == SFM_DOWNLOAD) &&
               (app_ui_data->rss_max_download >0 )){
                //already refreshing.
                //somehow if we just call handle_download_stopped_by_user
                //then when switching to NR, it won't respond.So we have 2 cases. TODO: fix this.
                //handle_download_stopped_by_user should be used in both cases
                /*
                if (!app_ui_data->start_on_background)
                    app_ui_data->search_mode = SFM_INTERRUPTED;
                    //download_cancel_all();
                    //on_refresh_finished(FALSE, FALSE); //No banner from RSS, but signal the applet
                
                //regard it as a user pressed stop download button
                //g_timeout_add(10, refresh_stop_from_applet, retval);
                else 
                    handle_download_stopped_by_user();
                //on_refresh_finished(FALSE, FALSE);
                */
                gdk_threads_enter();
                handle_download_stopped_by_user();
                gdk_threads_leave();
           } else
                   rss_dbus_signal_applet
                    (OSSO_RSS_FEED_READER_REFRESH_PREVENTED_SIGNAL);
	    g_timer_reset(messageTimer);
              retval->type = DBUS_TYPE_INVALID;
           osso_rpc_free_val (retval);
           ULOG_DEBUG("tvh: dbus returned");
           return OSSO_OK;
       }

       // app_ui_data->iap_action = OSSO_IAP_REFRESH_NO_DIALOG;
       app_ui_data->iap_action = OSSO_IAP_REFRESH_ALL_NO_DIALOG;
       osso_iap_connect(/*timed ? */OSSO_IAP_ANY/* : OSSO_IAP_ASK*/,
             timed ? OSSO_IAP_TIMED_CONNECT :
             OSSO_IAP_REQUESTED_CONNECT, app_ui_data);
	}
	g_timer_reset(messageTimer);
       retval->type = DBUS_TYPE_INVALID;
       osso_rpc_free_val (retval);
       return OSSO_OK;
    } else
    if (g_ascii_strcasecmp(method, OSSO_RSS_FEED_READER_DISPLAY_SIGNAL)
        == 0) {
        filename =
        g_strdup_printf("%s" G_DIR_SEPARATOR_S ".osso_rss_feed_reader"
            G_DIR_SEPARATOR_S "headline_clicked", g_get_home_dir());
        if (g_file_test(filename, G_FILE_TEST_EXISTS)) {
           g_unlink(filename);
        }
        g_free(filename);    
        
        feed = (gchar *) g_array_index(arguments, osso_rpc_t, 0).value.s;
        nr = g_array_index(arguments, osso_rpc_t, 1).value.u;
       
        /* If the news reader is running and is put into the background
        a state save is made (by calling callbacks.c:on_untop). Now when the 
        user clicks on a headline  After that, 
        the application is put to foreground again and the callbacks.c:on_top
        is called. This restores the saved state and displays the saved feed
        and the headline that was displayed is gone. For this reason,
        the state is not loaded when a headline is clicked */
       
        reset_state_file(app_data->osso);
            
        ui_feedlist_display_news(feed, nr);
    
        gtk_window_present(GTK_WINDOW(app_ui_data->app));
    
        retval->type = DBUS_TYPE_INVALID;
        osso_rpc_free_val (retval);
        return OSSO_OK;
    }

    else if (g_ascii_strcasecmp(method, OSSO_RSS_FEED_READER_ADD_SIGNAL)
        == 0) {

        ULOG_DEBUG("dbus msg OSSO_RSS_FEED_READER_ADD_AND_DISPLAY_SIGNAL");
           gchar * fd_source = (gchar *) g_array_index(arguments, osso_rpc_t, 0).value.s;

        ULOG_DEBUG("dbus_msg_handler: fd_source = %s", fd_source);

        /* Destroy those modal dialogs first */
        if (app_data->app_ui_data->importopmldialog != NULL)
            importopml_dialog_destroy();
        if (app_data->app_ui_data->add_feed_dialog != NULL)
            newdialog_destroy();
        if (app_data->app_ui_data->refresh_dialog != NULL)
        {
            gtk_widget_destroy(GTK_WIDGET(app_data->app_ui_data->refresh_dialog));
            app_data->app_ui_data->refresh_dialog = NULL;
            app_data->app_ui_data->dialog = NULL;
        }
        //manage folder dialog:
        //actually this is ok
        /*
        if (app_data->app_ui_data->dialog != NULL)
        {
            gtk_widget_destroy(GTK_WIDGET(app_data->app_ui_data->dialog));
            app_data->app_ui_data->dialog = NULL;
        }
        */

        // or invoke the Add_new_dialog and set the addressEntry 
        if (g_str_has_suffix( (const gchar *) fd_source, "opml") == TRUE)
        {
            ULOG_DEBUG("dbus_msg_handler: handling mime_open for .opml file");
            on_mime_open_add_opml_requested(app_data, fd_source);
            //g_free(fd_source);
            retval->type = DBUS_TYPE_INVALID;
            osso_rpc_free_val (retval);
            return OSSO_OK;
        }
        //Suppose it's a feed. Just add it. Might not be a feed though. note:
        //now that adding opml file from add-feed-dialog is ok, this should be fine
        else /*  (  g_str_has_suffix( (const gchar *) fd_source, "xml") ||
               g_str_has_suffix( (const gchar *) fd_source, "rss") ||
               g_str_has_suffix( (const gchar *) fd_source, "atom") )
        */
        {
            on_newbtn_clicked(app_data, fd_source);
            gtk_window_present(GTK_WINDOW(app_ui_data->app));
    
            retval->type = DBUS_TYPE_INVALID;
            //g_free(fd_source);// Needed?
            osso_rpc_free_val (retval);
            return OSSO_OK;
        }

    }
     else {
       ULOG_WARN("Unknown DBUS method: %s\n", method);
    }
    osso_rpc_free_val (retval);
    return OSSO_ERROR;
}
extern gboolean lifereaStarted;
/* Depending on the state of hw, do something */
void hw_event_handler(osso_hw_state_t * state, gpointer data)
{
    (void) data;

    //g_assert(state != NULL);
    g_return_if_fail ( state != NULL);
    
    ULOG_DEBUG("Hardware event received: %d %d %d %d", state->shutdown_ind,
               state->memory_low_ind, state->save_unsaved_data_ind, state->system_inactivity_ind);
    
    if (state->shutdown_ind) {
    /* Rebooting */
    //on_quit(FALSE);
    on_close(NULL, NULL,NULL);
    }
    
    if (state->memory_low_ind) {
    /* Memory low */
       /*If the page is rendering or downloading. We should stop it. If too many images
       for example: alienlovespredator.com/alp_rss.xml. It will crash
       */
       ULOG_DEBUG("*************************************************************************LOW MEMORY SIGNAL caught");
        //this will happen if NR is starting in low mem condition
        if (!lifereaStarted)
        {
            ULOG_DEBUG("News Reader hasn't started. Exiting now");
            exit(0);
        
        }       
        app_data->app_ui_data->low_mem = TRUE;
        app_data->app_ui_data->download_stopped = TRUE;
        gdk_threads_enter();
        gtkhtml_stop();
        handle_download_stopped_by_user();
        ULOG_DEBUG("Low memory signal handled");
        ui_show_lowmem_banner();
        gdk_threads_leave();    
       
    }
    else 
    {
        app_data->app_ui_data->low_mem = FALSE;    
    }
    
    if (state->save_unsaved_data_ind) {
    /* Save unsaved data. For this the itemlist must
       be emptied */
//     save_state(app_data);
//     ui_itemlist_search_init();
//         save_application_data();
//         ui_mainwindow_finish();
//     restore_state(app_data);
//         if(app_data == NULL)
//         return;
//         
//     if(app_data->app_ui_data == NULL)
//         return;
//         
//         if(app_data->app_ui_data->search_mode == SFM_NORMAL)
//     {
//             conf_feedlist_save();
//         ui_feedlist_do_for_all(NULL,
//                        ACTION_FILTER_FEED | ACTION_FILTER_DIRECTORY,
//                        feed_save);
// 
//             setBooleanConfValue(LAST_ITEMLIST_MODE, !itemlist_mode);
//     }    
    }
    if (state->system_inactivity_ind) {
    /* Minimum activity */
    }
}

void rss_iap_cb(struct iap_event_t *event, void *arg)
{
    /* Note: app_ui_data will be NULL if this is automatically generated! */
    /*AppUIData *app_ui_data = (AppUIData *) arg;*/
    AppUIData *app_ui_data;
    /* tvh 21May2006 : very hackish. note: We need a way to unset osso_iap_cb first */
    //if (!app_data)     
    //    return;
    //shouldn't happen as osso_iap_cb is already unset by setting to NULL
    g_return_if_fail(app_data!= NULL);
    //g_assert(app_data != NULL); 
   
    app_ui_data = app_data->app_ui_data;
    g_return_if_fail(app_ui_data!= NULL);
    //g_assert(app_ui_data != NULL);
    ULOG_DEBUG("rss_iap_cb: Connectivity event %d, app_ui_data %p", event->type,
           app_ui_data);

#ifdef WITHOUT_CONNECTIVITY
    /* If we've been compiled without connectivity, rig error into connect signal */
    if (OSSO_IAP_ERROR == event->type)
        event->type = OSSO_IAP_CONNECTED;
#endif
    
    if (OSSO_IAP_CONNECTED == event->type) 
    {
        ULOG_DEBUG("rss_iap_cb: CONNECTED");
        handle_iap_connected (event, app_ui_data);
    }
    else if (OSSO_IAP_ERROR == event->type)
    {
        ULOG_DEBUG("rss_iap_cb: ERROR");
        handle_iap_error (event, app_ui_data);
    }
    else if (OSSO_IAP_DISCONNECTED == event->type)
    {
        ULOG_DEBUG("rss_iap_cb: DISCONNECTED");
        handle_iap_disconnected (event, app_ui_data);
    }
}

/* Do initialization, create context, set topping callback,
   dbus-message handling callbaks, and hw-event callbacks. */
gboolean init_osso(AppData * app_data)
{
    osso_return_t ret;

    //g_assert(app_data != NULL);
    g_return_val_if_fail ((app_data != NULL), FALSE);
    //g_assert(app_data != NULL);
    
    /* gtk_init already started this . Calling this again will cause mem leaks */
    //g_type_init();

    ULOG_INFO("Initializing [%s] version [%s]", PACKAGE_NAME,
          PACKAGE_VERSION);
    app_data->osso =
    osso_initialize(PACKAGE_NAME, PACKAGE_VERSION, TRUE, NULL);

    //g_assert(app_data->osso);
    g_return_val_if_fail ((app_data->osso), FALSE); 
    if (app_data->osso == NULL) {
    ULOG_CRIT("Osso initialization failed");
    return FALSE;
    }
    //g_assert(app_data->osso);

    messageTimer = g_timer_new();
    /* Set topping callback */
    // tvh 20June2006 This func is deprecated. Should be handled by dbus in osso_rpc_set_cb
    /*
    osso_application_set_top_cb(app_data->osso,
                (osso_application_top_cb_f *)
                osso_top_callback, (gpointer) (app_data));
    */
    /* Set handling d-bus messages from session bus  */
    ret = osso_rpc_set_cb_f(app_data->osso,
                OSSO_RSS_FEED_READER_SERVICE,
                OSSO_RSS_FEED_READER_OBJECT_PATH,
                OSSO_RSS_FEED_READER_INTERFACE,
                dbus_req_handler, app_data);

    if (ret != OSSO_OK) {
    ULOG_CRIT("Could not set callback for receiving messages");
    }

    /* Set handling changes in HW states. Note: not tested */
    ret = osso_hw_set_event_cb(app_data->osso,
                   NULL, hw_event_handler, app_data);

    if (ret != OSSO_OK) {
    ULOG_CRIT("Could not set callback for HW monitoring");
    }

    ret = osso_iap_cb((osso_iap_cb_t) rss_iap_cb);

    if (ret != OSSO_OK) {
    ULOG_ERR("Could not set callback for connectivity");
    }

    return TRUE;
}

gboolean deinit_osso(AppData * app_data)
{
    //g_assert(app_data != NULL);
    //g_assert(app_data != NULL);
      g_return_val_if_fail ((app_data != NULL), FALSE);  
    ULOG_DEBUG("\n\n\n\ndeinit_osso\n\n\n\n");
    /* Unset callbacks */
    osso_application_unset_top_cb(app_data->osso, 
                        (osso_application_top_cb_f *) osso_top_callback, NULL);
    
    osso_rpc_unset_cb_f(app_data->osso,
            OSSO_RSS_FEED_READER_SERVICE,
            OSSO_RSS_FEED_READER_OBJECT_PATH,
            OSSO_RSS_FEED_READER_INTERFACE,
            dbus_req_handler, app_data);

    osso_hw_unset_event_cb(app_data->osso, NULL);
    //unset iap_cb
    osso_iap_cb(NULL);
    
    //g_assert(app_data->osso);
    g_return_val_if_fail ((app_data->osso), FALSE);
    osso_deinitialize(app_data->osso);
    g_timer_destroy(messageTimer);
    
    return TRUE;
}

gboolean open_url(gchar * url)
{
    ULOG_DEBUG(__FUNCTION__);

    gboolean reuseWindow =
    getBooleanConfValue(RSS_SETTINGS_REUSE_BROWSER_WINDOW);

    //g_assert(app_data != NULL);
    //g_assert(app_data != NULL);
    g_return_val_if_fail ((app_data != NULL), FALSE);

    osso_rpc_run_with_defaults(app_data->osso,
                   "osso_browser",
                   reuseWindow ? OSSO_BROWSER_LOAD_URL_REQ :
                   OSSO_BROWSER_OPEN_NEW_WINDOW_REQ, NULL,
                   DBUS_TYPE_STRING, url, DBUS_TYPE_INVALID);

    return TRUE;
}

gboolean rss_dbus_signal_applet(gchar * signal)
{
    DBusMessage *message = NULL;
    GError *err = NULL;
    osso_context_t *osso;
     
    //g_assert(app_data != NULL);
    //g_assert(app_data != NULL);
    g_return_val_if_fail ((app_data != NULL), FALSE);
    
    osso = app_data->osso;
    DBusConnection *connection = osso_get_dbus_connection(osso);
    ULOG_DEBUG(__FUNCTION__);

    message =
    dbus_message_new_signal
    (OSSO_RSS_FEED_READER_OBJECT_PATH,
     OSSO_RSS_FEED_READER_INTERFACE, signal);

    if (message == NULL) {
    ULOG_ERR("Failed to create signal %s", signal);
    }

    //dbus_message_set_auto_activation(message, FALSE);
    dbus_message_set_auto_start(message, FALSE); //new dbus v0.60 
    if (err == NULL && !(dbus_connection_send(connection, message, NULL))) {
       ULOG_ERR("Failed to send signal %s", signal);
    }

    if (message != NULL) {
    dbus_message_unref(message);
    }
    return TRUE;
}

gboolean rss_dbus_refresh_completed()
{
    return
    rss_dbus_signal_applet
    (OSSO_RSS_FEED_READER_REFRESH_FINISHED_SIGNAL);
}
