/**
    @file interface.c

    Dialogs and windows for the main application are created here.
*/


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <hildon-widgets/hildon-defines.h>
#include <hildon-widgets/hildon-caption.h>
#include <hildon-widgets/hildon-time-editor.h>
#include <hildon-widgets/hildon-find-toolbar.h>
#include <osso-log.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <langinfo.h>
#include <osso-helplib.h>

#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>

#include "main.h"
#include "common.h"
#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "appdata.h"
#include "ui_mainwindow.h"
#include "ui_feed_directory.h"
#include "ui_itemlist.h"
#include "gtkhtml/gtkhtml_plugin.h"

#define SCROLLED_WINDOW_FEEDLIST_SIZE 213
#define NEW_DIALOG_X_SIZE 400
#define REFRESH_DIALOG_X_SIZE 300
#define MANAGE_FOLDERS_DIALOG_XY_SIZE 300
#define CHANGE_FOLDER_DIALOG_XY_SIZE 300
#define PREF_DIALOG_X_SIZE 500
#define LABEL_MAX_WIDTH 23
#define PANED_GUTTER_SIZE 1
#define GUTTER_POSITION 200
#define MINIMAL_GUTTER_POSITION 50

extern GtkWidget *newspane;
extern GtkWidget *feedlist_container;
extern AppData *app_data;

/************************************************************************/
/*                        PRIVATE FUNCTIONS                             */
/************************************************************************/

/** Creates the find toolbar
  *
  * @param app_ui_data application specific data
  */
static void ui_create_find_toolbar(AppUIData * app_ui_data)
{
    HildonFindToolbar *find_toolbar = NULL;

    //g_assert(app_ui_data != NULL);
    //g_assert(app_ui_data->main_view != NULL);
    g_return_if_fail   (app_ui_data->main_view != NULL); 
    find_toolbar = HILDON_FIND_TOOLBAR
        (hildon_find_toolbar_new(_("rss_ti_find_toolbar")));
    //g_assert(find_toolbar != NULL);
    g_return_if_fail(find_toolbar != NULL);    
    app_ui_data->find_toolbar = find_toolbar;
    g_object_set(G_OBJECT(find_toolbar), "prefix", "", NULL);
    g_signal_connect(G_OBJECT(find_toolbar), "search",
                     G_CALLBACK(find_toolbar_search),
                      NULL);
    g_signal_connect(G_OBJECT(find_toolbar), "close",
                     G_CALLBACK(find_toolbar_close),
                     app_ui_data);
    gtk_box_pack_start(GTK_BOX(app_ui_data->main_view->vbox),
                       GTK_WIDGET(find_toolbar),
                       TRUE, TRUE, 0);
    gtk_widget_hide_all(GTK_WIDGET(find_toolbar));
}

static gchar* create_label_with_colon(gchar *str)
{
    if(str == NULL)
        return g_strdup("");
    
    return g_strconcat(str, ":", NULL);    
}

/************************************************************************/
/*                        PUBLIC FUNCTIONS                              */
/************************************************************************/

/** Selects the first item in the tree view
  *
  * @param tr the tree view
  */
gboolean tree_view_select_first(GtkWidget *tr)
{
    GtkTreeView *treeview = NULL;    

    if(tr == NULL)
        return FALSE;

    treeview = GTK_TREE_VIEW(tr);
    
    gtk_tree_view_set_cursor(treeview, gtk_tree_path_new_first(),
                     NULL, FALSE);
    return TRUE;
}

GtkWidget *create_mainwindow(AppUIData * app_ui_data)
{
    GtkWidget *mainwindow;

    //Not needed as we now use gtk_hpaned in stead of vbox
    //GtkWidget *vbox1;

    GtkWidget *menubar;

    GtkWidget *menuitem_feed;
    GtkWidget *menuitem_feed_menu;
    GtkWidget *menuitem_open;
    GtkWidget *menuitem_add;
    GtkWidget *menuitem_importopml;
    GtkWidget *menuitem_unsubscribe;
    GtkWidget *menuitem_properties;


    GtkWidget *menuitem_edit;
    GtkWidget *menuitem_edit_menu;
    GtkWidget *menuitem_cut;
    GtkWidget *menuitem_copy;
    GtkWidget *menuitem_paste;
    GtkWidget *menuitem_selectall;

    GtkWidget *menuitem_view;
    GtkWidget *menuitem_view_menu;
    GtkWidget *menuitem_folders;
    GtkWidget *menuitem_toolbar;
    GtkWidget *menuitem_toolbar_menu;
    GtkWidget *menuitem_full;
    GtkWidget *menuitem_normal;
    GtkWidget *menuitem_fullscreen;

    GtkWidget *menuitem_tools;
    GtkWidget *menuitem_tools_menu;
    GtkWidget *menuitem_settings;
    GtkWidget *menuitem_toolsmenu_separator1;
    GtkWidget *menuitem_find_on_page;
    GtkWidget *menuitem_manage_folders;
    GtkWidget *menuitem_refresh_feeds;
    GtkWidget *menuitem_toolsmenu_separator2;
    GtkWidget *menuitem_search_results;
    GtkWidget *menuitem_toolsmenu_separator3;
    GtkWidget *menuitem_help;
    GtkWidget *menuitem_feed_directory = NULL;

    GtkWidget *menuitem_close;

    //gchar *feed_directory_source = NULL;
    
    //g_assert(app_ui_data != NULL);
    ULOG_DEBUG("*** Begin create_mainwindow()");

    mainwindow = GTK_WIDGET(app_ui_data->main_view);

    menubar = GTK_WIDGET(hildon_appview_get_menu(app_ui_data->main_view));

    ULOG_DEBUG("*** creating vbox1");
    /*
    vbox1 = gtk_vbox_new(FALSE, 0);
    gtk_widget_show(vbox1);
    gtk_container_add(GTK_CONTAINER(mainwindow), vbox1);
    */
    ULOG_DEBUG("*** creating menubar - feed");

    menuitem_feed = gtk_menu_item_new_with_label(_("rss_me_feed"));
    gtk_widget_show(menuitem_feed);
    gtk_container_add(GTK_CONTAINER(menubar), menuitem_feed);

    menuitem_feed_menu = gtk_menu_new();
    gtk_menu_item_set_submenu(GTK_MENU_ITEM(menuitem_feed),
                  menuitem_feed_menu);

    menuitem_open = gtk_menu_item_new_with_label(_("rss_me_open"));
    gtk_widget_show(menuitem_open);
    gtk_container_add(GTK_CONTAINER(menuitem_feed_menu), menuitem_open);
    app_ui_data->menuitem_open = GTK_MENU_ITEM(menuitem_open);

    menuitem_add = gtk_menu_item_new_with_label(_("rss_me_add_a_feed"));
    gtk_widget_show(menuitem_add);
    gtk_container_add(GTK_CONTAINER(menuitem_feed_menu), menuitem_add);
    app_ui_data->menuitem_add = GTK_MENU_ITEM(menuitem_add);

    /*: Creating the importopml menu item under Menu->Feed
    FIXME: po string is missing in UI Spec
    */

    menuitem_importopml =
    gtk_menu_item_new_with_label(_("rss_me_import_opml"));
    gtk_widget_show(menuitem_importopml);
    gtk_container_add(GTK_CONTAINER(menuitem_feed_menu),
              menuitem_importopml);
    app_ui_data->menuitem_importopml =
    GTK_MENU_ITEM(menuitem_importopml);
    
    menuitem_unsubscribe =
    gtk_menu_item_new_with_label(_("rss_me_unsubscribe"));
    gtk_widget_show(menuitem_unsubscribe);
    gtk_container_add(GTK_CONTAINER(menuitem_feed_menu),
              menuitem_unsubscribe);
    app_ui_data->menuitem_unsubscribe =
    GTK_MENU_ITEM(menuitem_unsubscribe);

    menuitem_properties =
    gtk_menu_item_new_with_label(_("rss_me_edit_details"));
    gtk_widget_show(menuitem_properties);
    gtk_container_add(GTK_CONTAINER(menuitem_feed_menu),
              menuitem_properties);
    app_ui_data->menuitem_properties = GTK_MENU_ITEM(menuitem_properties);

    ULOG_DEBUG("*** creating menubar - edit");

    menuitem_edit = gtk_menu_item_new_with_label(_("rss_me_edit"));
    gtk_widget_show(menuitem_edit);
    gtk_container_add(GTK_CONTAINER(menubar), menuitem_edit);

    menuitem_edit_menu = gtk_menu_new();
    gtk_menu_item_set_submenu(GTK_MENU_ITEM(menuitem_edit),
                  menuitem_edit_menu);

    menuitem_cut = gtk_menu_item_new_with_label(_("rss_me_cut"));
    gtk_widget_show(menuitem_cut);
    gtk_container_add(GTK_CONTAINER(menuitem_edit_menu), menuitem_cut);
    app_ui_data->menuitem_cut = GTK_MENU_ITEM(menuitem_cut);
    gtk_widget_set_sensitive(GTK_WIDGET(menuitem_cut), FALSE);

    menuitem_copy = gtk_menu_item_new_with_label(_("rss_me_copy"));
    gtk_widget_show(menuitem_copy);
    gtk_container_add(GTK_CONTAINER(menuitem_edit_menu), menuitem_copy);
    app_ui_data->menuitem_copy = GTK_MENU_ITEM(menuitem_copy);
    gtk_widget_set_sensitive(GTK_WIDGET(menuitem_copy), FALSE);

    menuitem_paste = gtk_menu_item_new_with_label(_("rss_me_paste"));
    gtk_widget_show(menuitem_paste);
    gtk_container_add(GTK_CONTAINER(menuitem_edit_menu), menuitem_paste);
    app_ui_data->menuitem_paste = GTK_MENU_ITEM(menuitem_paste);
    gtk_widget_set_sensitive(GTK_WIDGET(menuitem_paste), FALSE);

    menuitem_selectall = gtk_menu_item_new_with_label(dgettext("osso-notes", "note_me_menu_edit_select_all"));
    gtk_widget_show(menuitem_selectall);
    gtk_container_add(GTK_CONTAINER(menuitem_edit_menu), menuitem_selectall);
    app_ui_data->menuitem_selectall = GTK_MENU_ITEM(menuitem_selectall);
    gtk_widget_set_sensitive(GTK_WIDGET(menuitem_selectall), FALSE);

    ULOG_DEBUG("*** creating menubar - view");

    menuitem_view = gtk_menu_item_new_with_label(_("rss_me_view"));
    gtk_widget_show(menuitem_view);
    gtk_container_add(GTK_CONTAINER(menubar), menuitem_view);

    menuitem_view_menu = gtk_menu_new();
    gtk_menu_item_set_submenu(GTK_MENU_ITEM(menuitem_view),
                  menuitem_view_menu);

    menuitem_folders =
    gtk_check_menu_item_new_with_label(_("rss_me_folders"));
    gtk_widget_show(menuitem_folders);
    gtk_container_add(GTK_CONTAINER(menuitem_view_menu), menuitem_folders);
    app_ui_data->menuitem_folders = GTK_CHECK_MENU_ITEM(menuitem_folders);

    menuitem_fullscreen =
    gtk_check_menu_item_new_with_label(_("rss_me_full_screen"));
    gtk_widget_show(menuitem_fullscreen);
    gtk_container_add(GTK_CONTAINER(menuitem_view_menu),
              menuitem_fullscreen);
    app_ui_data->menuitem_fullscreen =
    GTK_CHECK_MENU_ITEM(menuitem_fullscreen);

    menuitem_toolbar = gtk_menu_item_new_with_label(_("rss_me_toolbar"));
    gtk_widget_show(menuitem_toolbar);
    gtk_container_add(GTK_CONTAINER(menuitem_view_menu), menuitem_toolbar);

    menuitem_toolbar_menu = gtk_menu_new();
    gtk_menu_item_set_submenu(GTK_MENU_ITEM(menuitem_toolbar),
                  menuitem_toolbar_menu);

    menuitem_normal =
    gtk_check_menu_item_new_with_label(_("rss_me_normalscreen"));
    gtk_widget_show(menuitem_normal);
    gtk_container_add(GTK_CONTAINER(menuitem_toolbar_menu),
              menuitem_normal);
    app_ui_data->menuitem_normal = GTK_CHECK_MENU_ITEM(menuitem_normal);

    menuitem_full =
    gtk_check_menu_item_new_with_label(_("rss_me_fullscreen"));
    gtk_widget_show(menuitem_full);
    gtk_container_add(GTK_CONTAINER(menuitem_toolbar_menu), menuitem_full);
    app_ui_data->menuitem_full = GTK_CHECK_MENU_ITEM(menuitem_full);

    ULOG_DEBUG("*** creating menubar - tools");

    menuitem_tools = gtk_menu_item_new_with_label(_("rss_me_tools"));
    gtk_widget_show(menuitem_tools);
    gtk_container_add(GTK_CONTAINER(menubar), menuitem_tools);

    menuitem_tools_menu = gtk_menu_new();
    gtk_menu_item_set_submenu(GTK_MENU_ITEM(menuitem_tools),
                  menuitem_tools_menu);

    menuitem_settings = gtk_menu_item_new_with_label(_("rss_me_settings"));
    gtk_widget_show(menuitem_settings);
    gtk_container_add(GTK_CONTAINER(menuitem_tools_menu),
              menuitem_settings);

    menuitem_toolsmenu_separator1 = gtk_separator_menu_item_new();
    gtk_widget_show(menuitem_toolsmenu_separator1);
    gtk_container_add(GTK_CONTAINER(menuitem_tools_menu),
              menuitem_toolsmenu_separator1);
    gtk_widget_set_sensitive(menuitem_toolsmenu_separator1, FALSE);

    menuitem_find_on_page =
    gtk_check_menu_item_new_with_label(_("rss_me_find"));
    gtk_widget_show(menuitem_find_on_page);
    gtk_container_add(GTK_CONTAINER(menuitem_tools_menu),
              menuitem_find_on_page);
    app_ui_data->menuitem_find_on_page =
    GTK_MENU_ITEM(menuitem_find_on_page);              

    menuitem_manage_folders =
    gtk_menu_item_new_with_label(_("rss_me_manage_folders"));
    gtk_widget_show(menuitem_manage_folders);
    gtk_container_add(GTK_CONTAINER(menuitem_tools_menu),
              menuitem_manage_folders);
    app_ui_data->menuitem_manage_folders = GTK_MENU_ITEM(menuitem_manage_folders);

    menuitem_refresh_feeds =
    gtk_menu_item_new_with_label(_("rss_me_refresh_feeds"));
    gtk_widget_show(menuitem_refresh_feeds);
    gtk_container_add(GTK_CONTAINER(menuitem_tools_menu),
              menuitem_refresh_feeds);
    app_ui_data->menuitem_refresh_feeds =
    GTK_MENU_ITEM(menuitem_refresh_feeds);

    /*
    feed_directory_source = getStringConfValue(FEED_DIRECTORY_SOURCE);        
    
    if(strcmp(feed_directory_source, ""))
    {
    */
        menuitem_feed_directory =
        gtk_menu_item_new_with_label(_("rss_me_feed_directory"));
        gtk_widget_show(menuitem_feed_directory);
        gtk_container_add(GTK_CONTAINER(menuitem_tools_menu),
                  menuitem_feed_directory);
        app_ui_data->menuitem_feed_directory =
            GTK_MENU_ITEM(menuitem_feed_directory);

    //}

   // g_free(feed_directory_source);
    
    menuitem_toolsmenu_separator2 = gtk_separator_menu_item_new();
    gtk_widget_show(menuitem_toolsmenu_separator2);
    gtk_container_add(GTK_CONTAINER(menuitem_tools_menu),
              menuitem_toolsmenu_separator2);
    gtk_widget_set_sensitive(menuitem_toolsmenu_separator2, FALSE);

    menuitem_search_results =
    gtk_menu_item_new_with_label(_("rss_me_search_results"));
    gtk_widget_show(menuitem_search_results);
    gtk_container_add(GTK_CONTAINER(menuitem_tools_menu),
              menuitem_search_results);
    gtk_widget_set_sensitive(GTK_WIDGET(menuitem_search_results), FALSE);
    app_ui_data->menuitem_search_results =
    GTK_MENU_ITEM(menuitem_search_results);

    menuitem_toolsmenu_separator3 = gtk_separator_menu_item_new();
    gtk_widget_show(menuitem_toolsmenu_separator3);
    gtk_container_add(GTK_CONTAINER(menuitem_tools_menu),
              menuitem_toolsmenu_separator3);
    gtk_widget_set_sensitive(menuitem_toolsmenu_separator3, FALSE);

    menuitem_help = gtk_menu_item_new_with_label(_("rss_me_help"));
    gtk_widget_show(menuitem_help);
    gtk_container_add(GTK_CONTAINER(menuitem_tools_menu), menuitem_help);

    ULOG_DEBUG("*** creating menubar - close");
    menuitem_close = gtk_menu_item_new_with_label(_("rss_me_close"));
    gtk_widget_show(menuitem_close);
    gtk_container_add(GTK_CONTAINER(menubar), menuitem_close);



    ULOG_DEBUG("*** creating newsbox");
    /* :newsbox should be a GtkPaned widget rather than a hbox */
    //newsbox = gtk_hbox_new(FALSE, 0);
    newsbox = gtk_hpaned_new ();
    gtk_widget_show(newsbox);
    //newsbox->handle_size = 10;
    gtk_container_set_border_width  (GTK_CONTAINER(newsbox),PANED_GUTTER_SIZE);
    gtk_container_add(GTK_CONTAINER(mainwindow), newsbox);

    //TODO: Reconsider if this vbox1 is needed
    //don't see why vbox1 is needed
    //gtk_box_pack_start(GTK_BOX(vbox1), newsbox, TRUE, TRUE, 0);


    ULOG_DEBUG("*** creating scrolledwindow_feedlist");

    GtkWidget *scrolledwindow_feedlist =
    gtk_scrolled_window_new(NULL, NULL);
    gtk_widget_show(scrolledwindow_feedlist);
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW
                   (scrolledwindow_feedlist),
                   GTK_POLICY_AUTOMATIC,
                   GTK_POLICY_AUTOMATIC);
    gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW
                    (scrolledwindow_feedlist),
                    GTK_SHADOW_NONE);
    //gtk_widget_set_size_request(scrolledwindow_feedlist, SCROLLED_WINDOW_FEEDLIST_SIZE, -1);
    gtk_widget_set_size_request(scrolledwindow_feedlist, MINIMAL_GUTTER_POSITION, -1);


    ULOG_DEBUG("*** creating feedlist_container");
    /* GtkPaned is used, so this is not needed */
    /*
    GtkWidget *vseparator = gtk_vseparator_new();
    gtk_widget_show(vseparator);
    */
    feedlist_container = gtk_hbox_new(FALSE, 0);
    gtk_widget_show(feedlist_container);
    gtk_box_pack_start(GTK_BOX(feedlist_container),
               scrolledwindow_feedlist, TRUE, TRUE, 0);
    //gtk_box_pack_end(GTK_BOX(feedlist_container), vseparator, FALSE, FALSE,
    //         0);
    /* GtkPaned now */
    //gtk_box_pack_start(GTK_BOX(newsbox), feedlist_container, FALSE, FALSE,
    //           0);
    gtk_paned_pack1(GTK_PANED(newsbox),feedlist_container,
                                                     TRUE,  /*resize-able*/
                                                     FALSE); /*shrink-able*/
                                                     
    //gtk_paned_set_position (GTK_PANED (newsbox), GUTTER_POSITION);
    if (app_data->app_ui_data->gutter_position > 0)
        gtk_paned_set_position (GTK_PANED (newsbox),
                (gint) app_data->app_ui_data->gutter_position);

    ULOG_DEBUG("*** creating feedlist");

    feedlist = gtk_tree_view_new();
    g_signal_connect(G_OBJECT(feedlist), "drag_data_received",
                     G_CALLBACK(ui_dnd_receive_drag_data), feedlist);
    g_signal_connect(G_OBJECT(feedlist), "drag_drop",
                     G_CALLBACK(ui_dnd_drag_drop), feedlist);
    gtk_widget_show(feedlist);
    gtk_container_add(GTK_CONTAINER(scrolledwindow_feedlist), feedlist);
    gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(feedlist), FALSE);
    gtk_tree_view_set_reorderable(GTK_TREE_VIEW(feedlist), TRUE);

    //g_assert(app_data->app_ui_data != NULL);
    
    ui_create_find_toolbar(app_data->app_ui_data);
    
    ULOG_DEBUG("*** g_signal_connect_ block begins");

    /* feed menu */

    g_signal_connect((gpointer) menuitem_open, "activate",
             G_CALLBACK(on_menu_open), NULL);

    g_signal_connect((gpointer) menuitem_open, "insensitive_press",
             G_CALLBACK(on_feed_insensitive), NULL);

    g_signal_connect((gpointer) menuitem_add, "activate",
             G_CALLBACK(on_menu_feed_new), NULL);
    g_signal_connect((gpointer) menuitem_add,  "insensitive_press",
             G_CALLBACK(on_menu_feed_new), NULL);

    g_signal_connect((gpointer) menuitem_unsubscribe, "activate",
             G_CALLBACK(on_menu_delete), NULL);

    g_signal_connect((gpointer) menuitem_unsubscribe, "insensitive_press",
             G_CALLBACK(on_menu_delete), NULL);
    /* :11.Jan.06*/
    g_signal_connect((gpointer) menuitem_importopml, "activate",
             G_CALLBACK(on_menu_importopml), NULL);

    g_signal_connect((gpointer) menuitem_importopml, "insensitive_press",
             G_CALLBACK(on_menu_importopml), NULL);


    g_signal_connect((gpointer) menuitem_properties, "activate",
             G_CALLBACK(on_menu_properties), NULL);

    g_signal_connect((gpointer) menuitem_properties, "insensitive_press",
             G_CALLBACK(on_menu_properties), NULL);

    /* edit menu */

    g_signal_connect((gpointer) menuitem_edit, "activate",
             G_CALLBACK(on_menuitem_edit), app_ui_data);

    g_signal_connect((gpointer) menuitem_cut, "activate",
             G_CALLBACK(on_cut), app_ui_data);

    g_signal_connect((gpointer) menuitem_cut, "insensitive_press",
             G_CALLBACK(on_cut), app_ui_data);
    //Interface changed for these
    g_signal_connect((gpointer) menuitem_copy, "activate",
             G_CALLBACK(on_copy), app_ui_data);

    g_signal_connect((gpointer) menuitem_copy, "insensitive_press",
             G_CALLBACK(on_copy), app_ui_data);

    g_signal_connect((gpointer) menuitem_paste, "activate",
             G_CALLBACK(on_paste), app_ui_data);

    g_signal_connect((gpointer) menuitem_paste, "insensitive_press",
             G_CALLBACK(on_paste), app_ui_data);

    g_signal_connect((gpointer) menuitem_selectall, "activate",
             G_CALLBACK(on_selectall), app_ui_data);

    g_signal_connect((gpointer) menuitem_selectall, "insensitive_press",
             G_CALLBACK(on_selectall), app_ui_data);

    g_signal_connect ((gpointer) menuitem_find_on_page, "activate",
                     G_CALLBACK (toggle_find_on_page),
                    NULL);

    /* view menu */

    g_signal_connect((gpointer) menuitem_folders, "activate",
             G_CALLBACK(toggle_folder_list), NULL);

    g_signal_connect((gpointer) menuitem_normal, "activate",
             G_CALLBACK(toggle_toolbar), NULL);

    g_signal_connect((gpointer) menuitem_full, "activate",
             G_CALLBACK(toggle_toolbar), NULL);

    g_signal_connect((gpointer) menuitem_fullscreen, "activate",
             G_CALLBACK(on_toggle_fullscreen_mode), NULL);

    /* tools menu */
    /* Added this as feed directory might need to be insensitivized if there 
     * hasn't been any opml action yet
    */
    g_signal_connect((gpointer) menuitem_tools, "activate",
             G_CALLBACK(on_menuitem_tools), app_ui_data);

    g_signal_connect((gpointer) menuitem_refresh_feeds, "activate",
             G_CALLBACK(on_refresh_menuitem_selected),
             app_ui_data);

    g_signal_connect((gpointer) menuitem_refresh_feeds,
             "insensitive_press",
             G_CALLBACK(on_refresh_menuitem_selected),
             app_ui_data);

    g_signal_connect((gpointer) menuitem_manage_folders, "activate",
             G_CALLBACK(on_managefoldersbtn_clicked), NULL);
    g_signal_connect((gpointer) menuitem_manage_folders, "insensitive_press",
             G_CALLBACK(on_managefoldersbtn_clicked), NULL);

    g_signal_connect((gpointer) menuitem_settings, "activate",
             G_CALLBACK(on_prefbtn_clicked), NULL);

    g_signal_connect((gpointer) menuitem_search_results, "activate",
             G_CALLBACK(on_searchresults_menuitem_selected), NULL);

    g_signal_connect((gpointer) menuitem_search_results,
             "insensitive_press",
             G_CALLBACK(on_searchresults_menuitem_selected), NULL);

    g_signal_connect((gpointer) menuitem_help, "activate",
             G_CALLBACK(ui_help_cb), NULL);

    if(menuitem_feed_directory != NULL)
    {
        g_signal_connect((gpointer) menuitem_feed_directory, "activate",
                 G_CALLBACK(on_menuitem_feed_directory_clicked), NULL); 
        g_signal_connect((gpointer) menuitem_feed_directory, "insensitive_press",
                 G_CALLBACK(on_menuitem_feed_directory_clicked), NULL); 
    }

    /* close menu */

    g_signal_connect((gpointer) menuitem_close, "activate",
             G_CALLBACK(on_close), NULL);
    /* others */

    g_signal_connect((gpointer)app_data->app_ui_data->app, "delete_event",
                     G_CALLBACK(on_close), NULL);
    /*                     
    g_signal_connect((gpointer) mainwindow, "delete_event",
             G_CALLBACK(on_close), NULL);
    */             
    g_signal_connect((gpointer) mainwindow, "window_state_event",
             G_CALLBACK(on_mainwindow_window_state_event), NULL);
    g_signal_connect((gpointer) feedlist, "button_press_event",
             G_CALLBACK(on_mainfeedlist_button_press_event), NULL);
    g_signal_connect((gpointer) feedlist, "drag_end",
             G_CALLBACK(on_feedlist_drag_end), NULL);
    g_signal_connect((gpointer) feedlist, "drag_begin",
             G_CALLBACK(on_feedlist_drag_begin), NULL);             

    /* Activate the Folders check menu item. */
    ui_set_folders_menu_item_state(app_ui_data, TRUE);

    /* Activate the Toolbar check menu item. */
    ui_set_toolbar_normal_menu_item_state(app_ui_data, TRUE);

    /* Activate the Toolbar check menu item. */
    ui_set_toolbar_full_menu_item_state(app_ui_data, TRUE);

    ULOG_DEBUG("*** gtk_widget_grab_focus");

    gtk_widget_grab_focus(feedlist);

    return newsbox;
    //return vbox1;
}

gboolean tree_view_cycle_up(GtkWidget * tr)
{
    GtkTreeIter iter;
    GtkTreePath *path = NULL;
    GtkTreeModel *model = NULL;
    GtkTreeView *treeview = NULL;

    if (tr == NULL)
    return FALSE;

    treeview = GTK_TREE_VIEW(tr);
    model = gtk_tree_view_get_model(treeview);

    gtk_tree_view_get_cursor(GTK_TREE_VIEW(tr), &path, NULL);

    if (path == NULL || model == NULL)
    return FALSE;

    if (gtk_tree_path_compare(path, gtk_tree_path_new_root()) == 0) {
    gtk_tree_model_get_iter(model, &iter, gtk_tree_path_new_root());

    gboolean found = FALSE;
    while (!found) {
        gboolean check_next = TRUE;
        while (check_next) {
        path = gtk_tree_model_get_path(model, &iter);
        check_next = gtk_tree_model_iter_next(model, &iter);
        }

        if (gtk_tree_view_row_expanded(treeview, path)) {
        gtk_tree_path_down(path);
        gtk_tree_model_get_iter(model, &iter, path);
        } else
        found = TRUE;
    }
    gtk_tree_view_set_cursor(treeview, path, NULL, FALSE);
    return TRUE;
    }
    return FALSE;
}

gboolean tree_view_cycle_down(GtkWidget * tr)
{
    int toplevel_tree_size = 0;
    int *indices = NULL;
    GtkTreeIter iter;
    GtkTreePath *path = NULL;
    GtkTreeModel *model = NULL;
    GtkTreeView *treeview = NULL;

    if (tr == NULL)
    return FALSE;

    treeview = GTK_TREE_VIEW(tr);
    model = gtk_tree_view_get_model(treeview);

    gtk_tree_view_get_cursor(GTK_TREE_VIEW(tr), &path, NULL);

    if (path == NULL)
    return FALSE;

    gtk_tree_model_get_iter(model, &iter, path);

    toplevel_tree_size = gtk_tree_model_iter_n_children(model, NULL);
    indices = gtk_tree_path_get_indices(path);

    if (indices == NULL || indices[0] != (toplevel_tree_size - 1))
    return FALSE;

    if (!gtk_tree_model_iter_next(model, &iter)) {
    if (!gtk_tree_view_row_expanded(treeview, path)) {
        gtk_tree_view_set_cursor(treeview, gtk_tree_path_new_first(),
                     NULL, FALSE);
        return TRUE;
    }
    }
    return FALSE;
}
gint key_press_nr = 0;

gboolean key_press(GtkWidget * widget, GdkEventKey * event,
             AppUIData * app_ui_data)
{
    ULOG_DEBUG("Key press received");
    
    if(event->state & GDK_CONTROL_MASK)
    {
        ULOG_DEBUG("CONTROL");
        return TRUE;
    }
    
    if(event->state & GDK_SHIFT_MASK)
    {
        ULOG_DEBUG("SHIFT");
        return TRUE;
    }
    
    if(event->state & GDK_MOD1_MASK)
    {
        ULOG_DEBUG("MOD1");
        return TRUE;
    }
    
    if(event->state & GDK_MOD3_MASK)
    {
        ULOG_DEBUG("MOD3");
        return TRUE;
    }
    
    if(event->state & GDK_MOD4_MASK)
    {
        ULOG_DEBUG("MOD4");
        return TRUE;
    }
    
    if(event->state & GDK_MOD5_MASK)
    {
        ULOG_DEBUG("MOD5");
        return TRUE;
    }
    
    /*if (event->state & (GDK_CONTROL_MASK |
            GDK_SHIFT_MASK |
            GDK_MOD1_MASK |
            GDK_MOD3_MASK | GDK_MOD4_MASK | GDK_MOD5_MASK)) {
    return TRUE;
    }*/
    
    ULOG_DEBUG("Got a GDK_KEY_PRESS: %d", event->keyval);
    switch (event->keyval) 
    {
        case GDK_Left:
    if (GTK_WIDGET_VISIBLE(feedlist)) 
    {
        gtk_widget_grab_focus(feedlist);
    
        return TRUE;
    }
    break;
    
    case GDK_Right:
        if (app_ui_data->feed_displayed &&
        gtk_window_get_focus(GTK_WINDOW(app_ui_data->app)) !=
            //ui_htmlview_get_buffer())
        ui_mainwindow_get_active_htmlview())
    {
        //gtk_widget_grab_focus(ui_htmlview_get_buffer());
        gtk_widget_grab_focus(gtk_bin_get_child(GTK_BIN (ui_mainwindow_get_active_htmlview())));
    }
    return TRUE;
    break;
    
    case GDK_Up:
    if (gtk_window_get_focus(GTK_WINDOW(app_ui_data->app)) ==
        feedlist)
        return tree_view_cycle_up(feedlist);
    else 
    {        
        ULOG_DEBUG("<<<key-press: On the page now ");    
        key_press_nr ++;
        //if up_key_press_nr == 1) we will handle in release-key
        if (key_press_nr >= 2)
        {
            ULOG_DEBUG("key-press: YOU  are pressing a LONG DOWN KEY\n\
            There fore i'm scrolling UP");
            // by returning FALSE. Let the system scroll
            return FALSE;
        }
    }

    break;
    
    case GDK_Down:
    if (gtk_window_get_focus(GTK_WINDOW(app_ui_data->app)) ==
        feedlist)
        return tree_view_cycle_down(feedlist);


    else 
    {        
        ULOG_DEBUG("<<<key-press: On the page now ");    
        key_press_nr ++;
        //if down_key_press_nr == 1) we will handle in release-key
        if (key_press_nr >= 2)
        {
            ULOG_DEBUG("key-press: YOU  are pressing a LONG DOWN KEY\n\
            There fore i'm scrolling down");
            // by returning FALSE.Let the system scroll"
            return FALSE;
        }
    }
    break;
    
    case GDK_F4:
        ULOG_DEBUG("HW menu key pressed");
        gtk_widget_show_all(GTK_WIDGET(hildon_appview_get_menu(app_ui_data->main_view)));
    return TRUE;

    break;

    case GDK_F6: //Full screen
    key_press_nr ++;
    if (key_press_nr == 1)
        toggle_fullscreen_and_update_menu(widget, app_ui_data);
    return TRUE;
    break;


    
/*
    case GDK_F7:
        gtkhtml_zoom_in();
        return TRUE;
    break;

    case GDK_F8:
        gtkhtml_zoom_out();
        return TRUE;
    break;
*/
    default:
    break;
   }
   return FALSE;
}
gboolean key_release(GtkWidget * widget, GdkEventKey * event,
             AppUIData * app_ui_data)
{
    ULOG_DEBUG("Key release event received");

    nodePtr ptr = NULL;
    
    if(app_ui_data->dialog_closed)
    {
        ULOG_DEBUG("Dialog was closed. Ignoring the key-release signal");
        app_ui_data->dialog_closed = FALSE;
        return TRUE;
    }

    if(event->state & GDK_CONTROL_MASK)
    {
        ULOG_DEBUG("CONTROL");
        return TRUE;
    }
    
    if(event->state & GDK_SHIFT_MASK)
    {
        ULOG_DEBUG("SHIFT");
        return TRUE;
    }
    
    if(event->state & GDK_MOD1_MASK)
    {
        ULOG_DEBUG("MOD1");
        return TRUE;
    }
    
    if(event->state & GDK_MOD3_MASK)
    {
        ULOG_DEBUG("MOD3");
        return TRUE;
    }
    
    if(event->state & GDK_MOD4_MASK)
    {
        ULOG_DEBUG("MOD4");
        return TRUE;
    }
    
    if(event->state & GDK_MOD5_MASK)
    {
        ULOG_DEBUG("MOD5");
        return TRUE;
    }
    
    /*if (event->state & (GDK_CONTROL_MASK |
            GDK_SHIFT_MASK |
            GDK_MOD1_MASK |
            GDK_MOD3_MASK | GDK_MOD4_MASK | GDK_MOD5_MASK)) {
    return TRUE;
    }*/

    ULOG_DEBUG("Got an event != GDK_KEY_PRESS: %d", event->keyval);
    
    switch (event->keyval) {
    
    case GDK_Escape:
    ULOG_DEBUG("Escape hit with keyval: %d", event->keyval);
    if (SFM_NORMAL != app_ui_data->search_mode) {
        ULOG_DEBUG("SFM_NORMAL != search_mode: %d", app_ui_data->search_mode);
    if (SFM_SEARCH == app_ui_data->search_mode) {
        ULOG_DEBUG("Interrupting search");
        stop_search();
        //app_ui_data->search_mode = SFM_INTERRUPTED;
    } else if (SFM_DOWNLOAD == app_ui_data->search_mode) {
        /*
        ULOG_DEBUG("Interrupting refresh");
        app_ui_data->search_mode = SFM_INTERRUPTED;
        download_cancel_all();
        */
        handle_download_stopped_by_user();
//         on_refresh_finished(FALSE, FALSE);
    }
    return FALSE;
    /* Actually not TRUE */
    } else if (hildon_appview_get_fullscreen(hildon_app_get_appview(app_ui_data->app)) != 0) {
       toggle_fullscreen_and_update_menu(widget, app_ui_data);
       return FALSE;
    }
    break;
    
    case GDK_Return:
    ptr = (nodePtr) ui_feedlist_get_selected();

    if (gtk_window_get_focus(GTK_WINDOW(app_ui_data->app)) ==
        feedlist) {
        if (ptr && ((FST_FEED == ptr->type)
        || (FST_VFOLDER == ptr->type))) {
            ui_feedlist_load_selected(ptr);
        return TRUE;
        }
    }
    break;

    case GDK_Up:
    if (gtk_window_get_focus(GTK_WINDOW(app_ui_data->app)) ==
        feedlist)
        return TRUE;
    else  if (!GTK_IS_EDITABLE(gtk_window_get_focus(GTK_WINDOW(app_ui_data->app))) )
    {        
        ULOG_DEBUG("....key-release: On the page now ");    
        //down_key_press_nr --;
        //we had only 1 short press in release-key
        if (key_press_nr == 1)
        {
            ULOG_DEBUG("key-release: You made a SHORT Up key press. So i'm jumping UP");
            gtkhtml_jump_to_next_focusable_object(FALSE);
        }
        else
        {
          gtkhtml_grab_cond_focus(FALSE);
        }
        //reset counter
        key_press_nr = 0;
        //yes we handled it
        return TRUE;
    }


    case GDK_Down:
    if (gtk_window_get_focus(GTK_WINDOW(app_ui_data->app)) ==
        feedlist)
        return TRUE;
    //else if not GTKeditable, it should be gtkhtml focused
    else  if (!GTK_IS_EDITABLE(gtk_window_get_focus(GTK_WINDOW(app_ui_data->app))) )

    {        
        ULOG_DEBUG("<<key-press: On the page now ");    
        //down_key_press_nr --;
        //we had only 1 short press in release-key
        if (key_press_nr == 1)
        {
            ULOG_DEBUG("key-release: You made a SHORT Down key press. So i'm jumping Down");
            gtkhtml_jump_to_next_focusable_object(TRUE);
        }
        else
        {
          gtkhtml_grab_cond_focus(TRUE);
        }
        //reset counter
        key_press_nr = 0;
        //yes we handled it
        return TRUE;
    }
    break;

    case GDK_F6: //Full screen
        key_press_nr = 0;
        return TRUE;
    break;

    default:
    break;
    }
    return FALSE;
}

/*   11.Jan.06
    JUST creating, no signals or any actions yet
*/

GtkWidget *create_importopmldialog(void)
{
    ULOG_DEBUG("Creating the import OPML dialog .....\n");

    GtkWidget *importopmldialog;
    GtkWidget *dialog_vbox13;
    GtkWidget *feedimportopmlTable;
    GtkWidget *folderLabel;
    GtkWidget *folderData;

    GtkWidget *titleLabel;
    GtkWidget *opmlAddressEntry;
    GtkWidget *dialog_action_area13;

    GtkWidget *importopml_cancel;
    GtkWidget *importopml_ok;
    GtkWidget *importopml_browse;
    GtkWidget *importopml_changefolder;

    GtkSizeGroup *group =
    GTK_SIZE_GROUP(gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL));

    importopmldialog = gtk_dialog_new();
    gtk_window_set_title(GTK_WINDOW(importopmldialog), _("rss_ti_import_opml"));

    //gtk_window_set_position(GTK_WINDOW(importopmldialog), GTK_WIN_POS_CENTER);
    gtk_window_set_type_hint(GTK_WINDOW(importopmldialog),
                 GDK_WINDOW_TYPE_HINT_DIALOG);
    gtk_dialog_set_has_separator(GTK_DIALOG(importopmldialog), FALSE);

    /*Not sure if this available yet */    
    
    ossohelp_dialog_help_enable(
            GTK_DIALOG(importopmldialog),
            RSS_NEWS_READER_HELP_IMPORT_OPML_FILE,
            app_data->osso);
    
    /* create vbox*/
    dialog_vbox13 = GTK_DIALOG(importopmldialog)->vbox;
    gtk_widget_show(dialog_vbox13);

    feedimportopmlTable = gtk_vbox_new(FALSE, 0);
    gtk_widget_show(feedimportopmlTable);
    gtk_container_add(GTK_CONTAINER(dialog_vbox13), feedimportopmlTable);

    /*create this text here
    msgid "rss_ia_import_opml"
    msgstr “Write address to OPML file or .xml file or select a local file by tapping Browse.”
    */
    /*Opml file Address text box */

    opmlAddressEntry = gtk_entry_new();
    g_object_set(G_OBJECT(opmlAddressEntry), "autocap", FALSE, NULL);
    gtk_widget_show(opmlAddressEntry);
    gtk_entry_set_activates_default(GTK_ENTRY(opmlAddressEntry), TRUE);

    titleLabel = hildon_caption_new(group, _("rss_ia_feed_address"),
                    opmlAddressEntry, NULL,
                    HILDON_CAPTION_OPTIONAL);
    gtk_widget_show(titleLabel);
    gtk_box_pack_start_defaults(GTK_BOX(feedimportopmlTable), titleLabel);

    /* Feed Folder to contain the opml feeds */
    folderData = gtk_label_new("");
    gtk_misc_set_alignment(GTK_MISC(folderData), 0, 0.5);
    gtk_widget_show(folderData);

    folderLabel = hildon_caption_new(group, _("rss_ia_feed_folder"),
                     folderData, NULL,
                     HILDON_CAPTION_OPTIONAL);
    gtk_widget_show(folderLabel);
    gtk_box_pack_start_defaults(GTK_BOX(feedimportopmlTable), folderLabel);

    /* button OK */
    dialog_action_area13 = GTK_DIALOG(importopmldialog)->action_area;
    gtk_widget_show(dialog_action_area13);
    gtk_button_box_set_layout(GTK_BUTTON_BOX(dialog_action_area13),
                  GTK_BUTTONBOX_END);
    
    GtkRequisition r;
    gtk_widget_size_request(gtk_label_new(_("rss_bd_ok")), &r);

    importopml_ok = gtk_button_new_with_label(_("rss_bd_ok"));
    gtk_widget_show(importopml_ok);

    gtk_widget_set_size_request(GTK_WIDGET(importopml_ok),r.width+40,-1);
    gtk_dialog_add_action_widget(GTK_DIALOG(importopmldialog), importopml_ok,
                 GTK_RESPONSE_OK);
    GTK_WIDGET_SET_FLAGS(importopml_ok, GTK_CAN_DEFAULT);
     
    /* button BROWSE */
    importopml_browse = gtk_button_new_with_label(_("rss_bd_browse"));
    gtk_widget_show(importopml_browse);

    //     gtk_widget_set_size_request(GTK_WIDGET(importopml_browse),150,-1);
    gtk_dialog_add_action_widget(GTK_DIALOG(importopmldialog), importopml_browse,
                 RSS_RESPONSE_FEED_IMPORT_OPML_BROWSE);
    GTK_WIDGET_SET_FLAGS(importopml_browse, GTK_CAN_DEFAULT);

    /*button 'Change Folder' */
    importopml_changefolder =
     gtk_button_new_with_label(_("rss_bd_change_folder"));

    gtk_widget_show(importopml_changefolder);
    //gtk_widget_set_size_request(GTK_WIDGET(importopml_changefolder),150,-1);
    gtk_box_pack_start(GTK_BOX(dialog_action_area13), importopml_changefolder,
               FALSE, FALSE, 0);

    importopml_cancel = gtk_button_new_with_label(_("rss_bd_cancel"));
    gtk_widget_show(importopml_cancel);

    gtk_widget_size_request(gtk_label_new(_("rss_bd_cancel")), &r);
    gtk_widget_set_size_request(GTK_WIDGET(importopml_cancel),r.width+40,-1);

    /*button 'Cancel' */
    //gtk_widget_set_size_request(GTK_WIDGET(importopml_cancel),120,-1);
    gtk_dialog_add_action_widget(GTK_DIALOG(importopmldialog), importopml_cancel,
                 GTK_RESPONSE_CANCEL);
    GTK_WIDGET_SET_FLAGS(importopml_cancel, GTK_CAN_DEFAULT);    
    
    /* Store pointers to all widgets, for use by lookup_widget(). */
    GLADE_HOOKUP_OBJECT_NO_REF(importopmldialog, importopmldialog, "importopmldialog");
    GLADE_HOOKUP_OBJECT_NO_REF(importopmldialog, dialog_vbox13, "dialog_vbox13");
    GLADE_HOOKUP_OBJECT(importopmldialog, opmlAddressEntry, "opmlAddressEntry");

    GLADE_HOOKUP_OBJECT(importopmldialog, folderData, "folderName");
    GLADE_HOOKUP_OBJECT_NO_REF(importopmldialog, dialog_action_area13,
                   "dialog_action_area13");
    GLADE_HOOKUP_OBJECT(importopmldialog, importopml_cancel, "importopml_cancel");
    GLADE_HOOKUP_OBJECT(importopmldialog, importopml_ok, "importopml_ok");
    GLADE_HOOKUP_OBJECT(importopmldialog, importopml_browse, "importopml_browse");
    GLADE_HOOKUP_OBJECT(importopmldialog, importopml_changefolder, "changefolder");

    gtk_widget_grab_default(importopml_ok);
    return importopmldialog;

}
GtkWidget *create_propdialog(void)
{
    GtkWidget *propdialog;
    GtkWidget *dialog_vbox13;
    GtkWidget *feedPropTable;
    GtkWidget *folderLabel;
    GtkWidget *folderData;
    GtkWidget *subscribedLabel;
    GtkWidget *subscribedData;
    GtkWidget *titleLabel;
    GtkWidget *locationLabel;
    GtkWidget *feedNameEntry;
    GtkWidget *feedLocationEntry;
    GtkWidget *dialog_action_area13;
    GtkWidget *prop_cancel;
    GtkWidget *prop_ok;
    GtkWidget *prop_unsubscribe;
    GtkWidget *prop_changefolder;

    GtkSizeGroup *group =
    GTK_SIZE_GROUP(gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL));

    propdialog = gtk_dialog_new();
    gtk_window_set_title(GTK_WINDOW(propdialog), _("rss_ti_feed_details"));
//     gtk_window_set_position(GTK_WINDOW(propdialog), GTK_WIN_POS_CENTER);
    gtk_window_set_type_hint(GTK_WINDOW(propdialog),
                 GDK_WINDOW_TYPE_HINT_DIALOG);
    gtk_dialog_set_has_separator(GTK_DIALOG(propdialog), FALSE);
    
    ossohelp_dialog_help_enable(
            GTK_DIALOG(propdialog),
            RSS_NEWS_READER_HELP_DETAILS,
            app_data->osso);

    dialog_vbox13 = GTK_DIALOG(propdialog)->vbox;
    gtk_widget_show(dialog_vbox13);

    feedPropTable = gtk_vbox_new(FALSE, 0);
    gtk_widget_show(feedPropTable);
    gtk_container_add(GTK_CONTAINER(dialog_vbox13), feedPropTable);

    feedNameEntry = gtk_entry_new();
    g_object_set(G_OBJECT(feedNameEntry), "autocap", FALSE, NULL);
    gtk_widget_show(feedNameEntry);
    gtk_entry_set_activates_default(GTK_ENTRY(feedNameEntry), TRUE);

    titleLabel = hildon_caption_new(group, _("rss_ia_feed_title"),
                    feedNameEntry, NULL,
                    HILDON_CAPTION_OPTIONAL);
    gtk_widget_show(titleLabel);
    //gtk_box_pack_start_defaults(GTK_BOX(feedPropTable), titleLabel);
    gtk_box_pack_start(GTK_BOX(feedPropTable), titleLabel,FALSE,FALSE,2);
    
    feedLocationEntry = gtk_entry_new();
    g_object_set(G_OBJECT(feedLocationEntry), "autocap", FALSE, NULL);
    gtk_widget_show(feedLocationEntry);
    gtk_entry_set_activates_default(GTK_ENTRY(feedLocationEntry), TRUE);
    
    locationLabel = hildon_caption_new(group, _("rss_ia_feed_address"),
                    feedLocationEntry, NULL,
                    HILDON_CAPTION_OPTIONAL);
    gtk_widget_show(locationLabel);
    //gtk_box_pack_start_defaults(GTK_BOX(feedPropTable), locationLabel);
    gtk_box_pack_start(GTK_BOX(feedPropTable), locationLabel,FALSE,FALSE,2);

    subscribedData = gtk_label_new("-");
    gtk_misc_set_alignment(GTK_MISC(subscribedData), 0, 0.5);
    gtk_widget_show(subscribedData);

    subscribedLabel = hildon_caption_new(group, _("rss_ia_subscribed"),
                     subscribedData, NULL,
                     HILDON_CAPTION_OPTIONAL);
    gtk_widget_show(subscribedLabel);
    gtk_box_pack_start_defaults(GTK_BOX(feedPropTable), subscribedLabel);

    folderData = gtk_label_new("");
    gtk_misc_set_alignment(GTK_MISC(folderData), 0, 0.5);
    gtk_widget_show(folderData);

    folderLabel = hildon_caption_new(group, _("rss_ia_feed_folder"),
                     folderData, NULL,
                     HILDON_CAPTION_OPTIONAL);
    gtk_widget_show(folderLabel);
    gtk_box_pack_start_defaults(GTK_BOX(feedPropTable), folderLabel);

    dialog_action_area13 = GTK_DIALOG(propdialog)->action_area;
    gtk_widget_show(dialog_action_area13);
    gtk_button_box_set_layout(GTK_BUTTON_BOX(dialog_action_area13),
                  GTK_BUTTONBOX_END);
    
    GtkRequisition r;
    gtk_widget_size_request(gtk_label_new(_("rss_bd_ok")), &r);

    prop_ok = gtk_button_new_with_label(_("rss_bd_ok"));
    gtk_widget_show(prop_ok);

    gtk_widget_set_size_request(GTK_WIDGET(prop_ok),r.width+40,-1);
    gtk_dialog_add_action_widget(GTK_DIALOG(propdialog), prop_ok,
                 GTK_RESPONSE_OK);
    GTK_WIDGET_SET_FLAGS(prop_ok, GTK_CAN_DEFAULT);
     
    prop_unsubscribe = gtk_button_new_with_label(_("rss_bd_unsubscribe"));
    gtk_widget_show(prop_unsubscribe);

//     gtk_widget_set_size_request(GTK_WIDGET(prop_unsubscribe),150,-1);
    gtk_dialog_add_action_widget(GTK_DIALOG(propdialog), prop_unsubscribe,
                 RSS_RESPONSE_FEED_PROP_UNSUBSCRIBE);
    GTK_WIDGET_SET_FLAGS(prop_unsubscribe, GTK_CAN_DEFAULT);

    prop_changefolder =
     gtk_button_new_with_label(_("rss_bd_change_folder"));

    gtk_widget_show(prop_changefolder);
//     gtk_widget_set_size_request(GTK_WIDGET(prop_changefolder),150,-1);
    gtk_box_pack_start(GTK_BOX(dialog_action_area13), prop_changefolder,
               FALSE, FALSE, 0);

    prop_cancel = gtk_button_new_with_label(_("rss_bd_cancel"));
    gtk_widget_show(prop_cancel);

    gtk_widget_size_request(gtk_label_new(_("rss_bd_cancel")), &r);
    gtk_widget_set_size_request(GTK_WIDGET(prop_cancel),r.width+40,-1);
    
//     gtk_widget_set_size_request(GTK_WIDGET(prop_cancel),120,-1);
    gtk_dialog_add_action_widget(GTK_DIALOG(propdialog), prop_cancel,
                 GTK_RESPONSE_CANCEL);
    GTK_WIDGET_SET_FLAGS(prop_cancel, GTK_CAN_DEFAULT);    
    
    /* Store pointers to all widgets, for use by lookup_widget(). */
    GLADE_HOOKUP_OBJECT_NO_REF(propdialog, propdialog, "propdialog");
    GLADE_HOOKUP_OBJECT_NO_REF(propdialog, dialog_vbox13, "dialog_vbox13");
    GLADE_HOOKUP_OBJECT(propdialog, feedNameEntry, "feedNameEntry");
    GLADE_HOOKUP_OBJECT(propdialog, feedLocationEntry, "feedLocationEntry");
    GLADE_HOOKUP_OBJECT(propdialog, subscribedData, "subscribedData");
    GLADE_HOOKUP_OBJECT(propdialog, folderData, "folderName");
    GLADE_HOOKUP_OBJECT_NO_REF(propdialog, dialog_action_area13,
                   "dialog_action_area13");
    GLADE_HOOKUP_OBJECT(propdialog, prop_cancel, "prop_cancel");
    GLADE_HOOKUP_OBJECT(propdialog, prop_ok, "prop_ok");
    GLADE_HOOKUP_OBJECT(propdialog, prop_changefolder, "changefolder");

    gtk_widget_grab_default(prop_ok);
    return propdialog;
}

GtkWidget *create_newdialog(void)
{
    GtkWidget *newdialog;
    GtkWidget *dialog_vbox2;
    GtkWidget *sourceEntry;
    GtkWidget *label104;
    GtkWidget *dialog_action_area2;
    GtkWidget *cancelbtn;
    GtkWidget *changefolderbtn;
    GtkWidget *newfeedbtn;
    GtkWidget *labelFolder;
    GtkWidget *folderName;
    int length = 0;

    //g_assert(app_data != NULL);
    //g_assert(app_data->app_ui_data != NULL);
    
    GtkSizeGroup *group =
    GTK_SIZE_GROUP(gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL));

    newdialog = gtk_dialog_new();
    gtk_container_set_border_width(GTK_CONTAINER(newdialog), 6);
    gtk_window_set_title(GTK_WINDOW(newdialog), _("rss_ti_add_a_feed"));
//     gtk_window_set_position(GTK_WINDOW(newdialog), GTK_WIN_POS_CENTER);
    gtk_window_set_default_size(GTK_WINDOW(newdialog), NEW_DIALOG_X_SIZE, -1);
    gtk_window_set_type_hint(GTK_WINDOW(newdialog),
                 GDK_WINDOW_TYPE_HINT_DIALOG);
    gtk_dialog_set_has_separator(GTK_DIALOG(newdialog), FALSE);

    ossohelp_dialog_help_enable(
            GTK_DIALOG(newdialog),
            RSS_NEWS_READER_HELP_ADD_FEED,
            app_data->osso);

    dialog_vbox2 = GTK_DIALOG(newdialog)->vbox;
    gtk_widget_show(dialog_vbox2);

    sourceEntry = gtk_entry_new();
    g_object_set(G_OBJECT(sourceEntry), "autocap", FALSE, NULL);
    gtk_widget_show(sourceEntry);

    label104 = hildon_caption_new(group, _("rss_ia_feed_address"),
                  sourceEntry, NULL,
                  HILDON_CAPTION_OPTIONAL);
    gtk_widget_show(label104);
    gtk_box_pack_start_defaults(GTK_BOX(dialog_vbox2), label104);

    folderName = gtk_label_new(_("rss_ia_news_feed_folders"));
    length = g_utf8_strlen(_("rss_ia_news_feed_folders"), -1);
    if(length > LABEL_MAX_WIDTH)
         gtk_label_set_max_width_chars(GTK_LABEL(folderName), length);
    else
         gtk_label_set_max_width_chars(GTK_LABEL(folderName), LABEL_MAX_WIDTH);
    gtk_label_set_ellipsize(GTK_LABEL(folderName), PANGO_ELLIPSIZE_END);
    gtk_widget_show(folderName);
    gtk_misc_set_alignment(GTK_MISC(folderName), 0, 0.5);

    labelFolder = hildon_caption_new(group, _("rss_ia_add_to_folder"),
                     folderName, NULL,
                     HILDON_CAPTION_OPTIONAL);
    gtk_widget_show(labelFolder);
    gtk_box_pack_start_defaults(GTK_BOX(dialog_vbox2), labelFolder);

    dialog_action_area2 = GTK_DIALOG(newdialog)->action_area;
    gtk_widget_show(dialog_action_area2);
    gtk_button_box_set_layout(GTK_BUTTON_BOX(dialog_action_area2),
                  GTK_BUTTONBOX_END);

    app_data->app_ui_data->newfeedbtn = newfeedbtn = 
        gtk_button_new_with_label(_("rss_bd_ok"));
    gtk_widget_show(newfeedbtn);
    gtk_dialog_add_action_widget(GTK_DIALOG(newdialog), newfeedbtn,
                 GTK_RESPONSE_OK);
    GTK_WIDGET_SET_FLAGS(newfeedbtn, GTK_CAN_DEFAULT);

    app_data->app_ui_data->changefolderbtn = changefolderbtn = 
        gtk_button_new_with_label(_("rss_bd_change_folder"));
    gtk_widget_show(changefolderbtn);
    gtk_box_pack_start(GTK_BOX(dialog_action_area2), changefolderbtn,
               FALSE, FALSE, 0);
    GTK_WIDGET_SET_FLAGS(changefolderbtn, GTK_CAN_DEFAULT);

    app_data->app_ui_data->cancelbtn = cancelbtn = 
        gtk_button_new_with_label(_("rss_bd_cancel"));
    gtk_widget_show(cancelbtn);
    gtk_dialog_add_action_widget(GTK_DIALOG(newdialog), cancelbtn,
                 GTK_RESPONSE_CANCEL);
    GTK_WIDGET_SET_FLAGS(cancelbtn, GTK_CAN_DEFAULT);

    /* Store pointers to all widgets, for use by lookup_widget(). */

    GLADE_HOOKUP_OBJECT_NO_REF(newdialog, newdialog, "newdialog");
    GLADE_HOOKUP_OBJECT_NO_REF(newdialog, dialog_vbox2, "dialog_vbox2");
    GLADE_HOOKUP_OBJECT(newdialog, sourceEntry, "sourceEntry");
    GLADE_HOOKUP_OBJECT(newdialog, label104, "label104");
    GLADE_HOOKUP_OBJECT_NO_REF(newdialog, dialog_action_area2,
                   "dialog_action_area2");
    GLADE_HOOKUP_OBJECT(newdialog, cancelbtn, "cancelbtn");
    GLADE_HOOKUP_OBJECT(newdialog, newfeedbtn, "newfeedbtn");
    GLADE_HOOKUP_OBJECT(newdialog, folderName, "folderName");
    GLADE_HOOKUP_OBJECT(newdialog, changefolderbtn, "changefolder");

    return newdialog;
}

GtkWidget *create_refresh_dialog()
{
    GtkWidget *refresh_dialog;
    GtkWidget *refresh_label;
    GtkWidget *radio_button_all_feeds;
    GtkWidget *radio_button_selected_feed;
    GtkWidget *refresh_ok_button;
    GtkWidget *refresh_cancel_button;
    GtkWidget *vbox;
    GtkWidget *table;

    //g_assert(app_data != NULL);
    //g_assert(app_data->app_ui_data != NULL);
    
    refresh_dialog = app_data->app_ui_data->refresh_dialog = gtk_dialog_new();
    gtk_window_set_title(GTK_WINDOW(refresh_dialog),
             _("rss_ti_refresh_feeds"));
//     gtk_window_set_position(GTK_WINDOW(refresh_dialog),
//                 GTK_WIN_POS_CENTER);
    gtk_window_set_default_size(GTK_WINDOW(refresh_dialog), REFRESH_DIALOG_X_SIZE, -1);
    gtk_window_set_type_hint(GTK_WINDOW(refresh_dialog),
                 GDK_WINDOW_TYPE_HINT_DIALOG);
    gtk_window_set_transient_for(GTK_WINDOW(refresh_dialog),
                 GTK_WINDOW(app_data->app_ui_data->app));
    gtk_window_set_modal(GTK_WINDOW(refresh_dialog), TRUE);
    gtk_dialog_set_has_separator(GTK_DIALOG(refresh_dialog), FALSE);

    ossohelp_dialog_help_enable(
            GTK_DIALOG(refresh_dialog),
            RSS_NEWS_READER_HELP_REFRESH,
            app_data->osso);

    vbox = GTK_DIALOG(refresh_dialog)->vbox;
    gtk_widget_show(vbox);

    table = gtk_table_new(2, 2, FALSE);

    gtk_widget_show(table);

    gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, FALSE, 10);
    refresh_label = gtk_label_new(_("rss_ia_refresh"));
    gtk_widget_show(refresh_label);

    gtk_table_attach(GTK_TABLE(table), refresh_label, 0, 1, 0, 1, GTK_FILL,
             GTK_FILL, 10, 0);

    radio_button_all_feeds =
    gtk_radio_button_new_with_label(NULL, _("rss_ia_all_feeds"));
    gtk_widget_show(radio_button_all_feeds);
    GSList *rbgroup =
    gtk_radio_button_get_group(GTK_RADIO_BUTTON
                   (radio_button_all_feeds));
    radio_button_selected_feed =
    gtk_radio_button_new_with_label(rbgroup,
                    _("rss_ia_selected_feed_only"));
    gtk_widget_show(radio_button_selected_feed);

    gtk_table_attach(GTK_TABLE(table), radio_button_all_feeds, 1, 2, 0, 1,
             GTK_FILL, GTK_FILL, 10, 0);
    gtk_table_attach(GTK_TABLE(table), radio_button_selected_feed, 1, 2, 1,
             2, GTK_FILL, GTK_FILL, 10, 0);

    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio_button_all_feeds),
                 TRUE);

    refresh_ok_button = gtk_button_new_with_label(_("rss_bd_ok"));
    gtk_widget_show(refresh_ok_button);
    gtk_dialog_add_action_widget(GTK_DIALOG(refresh_dialog),
                 refresh_ok_button, GTK_RESPONSE_OK);

    refresh_cancel_button = gtk_button_new_with_label(_("rss_bd_cancel"));
    gtk_widget_show(refresh_cancel_button);
    gtk_dialog_add_action_widget(GTK_DIALOG(refresh_dialog),
                 refresh_cancel_button,
                 GTK_RESPONSE_CANCEL);

    g_signal_connect(G_OBJECT(refresh_dialog), "key-press-event",
             G_CALLBACK(key_press_for_cancelling_dialog), NULL);

    GTK_WIDGET_SET_FLAGS(refresh_ok_button, GTK_CAN_DEFAULT);

    GLADE_HOOKUP_OBJECT_NO_REF(refresh_dialog, refresh_dialog,
                   "refresh_dialog");
    GLADE_HOOKUP_OBJECT_NO_REF(refresh_dialog, vbox, "vbox");
    GLADE_HOOKUP_OBJECT(refresh_dialog, radio_button_selected_feed,
            "radio_button_selected_feed");

    return refresh_dialog;
}

GtkWidget *create_manage_folders_dialog(void)
{
    GtkWidget *mf_dialog;
    GtkWidget *dialog_vbox;
    GtkWidget *dialog_action_area;
    GtkWidget *mf_newbtn;
    GtkWidget *mf_renamebtn;
    GtkWidget *mf_deletebtn;
    GtkWidget *mf_closebtn;
    GtkWidget *folder_view;
    GtkWidget *folder_scrollwindow;

    ULOG_DEBUG(__FUNCTION__);

    //g_assert(app_data != NULL);
    //g_assert(app_data->app_ui_data != NULL);
    
    mf_dialog = gtk_dialog_new();
    gtk_window_set_title(GTK_WINDOW(mf_dialog),
             _("rss_ti_manage_folders"));
//     gtk_window_set_position(GTK_WINDOW(mf_dialog), GTK_WIN_POS_CENTER);
    gtk_window_set_default_size(GTK_WINDOW(mf_dialog), MANAGE_FOLDERS_DIALOG_XY_SIZE,
                                MANAGE_FOLDERS_DIALOG_XY_SIZE);
    gtk_window_set_type_hint(GTK_WINDOW(mf_dialog),
                 GDK_WINDOW_TYPE_HINT_DIALOG);
    gtk_window_set_transient_for(GTK_WINDOW(mf_dialog),
                 GTK_WINDOW(app_data->app_ui_data->app));
    gtk_window_set_modal(GTK_WINDOW(mf_dialog), TRUE);
    gtk_dialog_set_has_separator(GTK_DIALOG(mf_dialog), FALSE);

    ossohelp_dialog_help_enable(
            GTK_DIALOG(mf_dialog),
            RSS_NEWS_READER_HELP_MANAGE,
            app_data->osso);

    dialog_vbox = GTK_DIALOG(mf_dialog)->vbox;
    gtk_widget_show(dialog_vbox);

    folder_scrollwindow = gtk_scrolled_window_new(NULL, NULL);
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW
                   (folder_scrollwindow), GTK_POLICY_NEVER,
                   GTK_POLICY_AUTOMATIC);
    gtk_widget_show(folder_scrollwindow);

    folder_view = gtk_tree_view_new();
    gtk_widget_show(folder_view);
    gtk_container_add(GTK_CONTAINER(folder_scrollwindow), folder_view);
    gtk_container_add(GTK_CONTAINER(dialog_vbox), folder_scrollwindow);

    gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(folder_view), FALSE);
    gtk_tree_view_set_reorderable(GTK_TREE_VIEW(folder_view), TRUE);

    dialog_action_area = GTK_DIALOG(mf_dialog)->action_area;
    gtk_widget_show(dialog_action_area);
    gtk_button_box_set_layout(GTK_BUTTON_BOX(dialog_action_area),
                  GTK_BUTTONBOX_END);

    mf_newbtn = gtk_button_new_with_label(_("rss_bd_new"));
    gtk_widget_show(mf_newbtn);
    gtk_dialog_add_action_widget(GTK_DIALOG(mf_dialog), mf_newbtn,
                 RSS_RESPONSE_NEW_FOLDER);

    mf_renamebtn = gtk_button_new_with_label(_("rss_bd_rename"));
    gtk_widget_show(mf_renamebtn);
    gtk_dialog_add_action_widget(GTK_DIALOG(mf_dialog), mf_renamebtn,
                 RSS_RESPONSE_RENAME_FOLDER);

    mf_deletebtn = gtk_button_new_with_label(_("rss_bd_delete"));
    gtk_widget_show(mf_deletebtn);
    gtk_dialog_add_action_widget(GTK_DIALOG(mf_dialog), mf_deletebtn,
                 RSS_RESPONSE_DELETE_FOLDER);

    mf_closebtn = gtk_button_new_with_label(_("rss_bd_close"));
    gtk_widget_show(mf_closebtn);
    gtk_dialog_add_action_widget(GTK_DIALOG(mf_dialog), mf_closebtn,
                 GTK_RESPONSE_CANCEL);

    g_signal_connect(G_OBJECT(mf_dialog), "key-press-event",
             G_CALLBACK(key_press_for_cancelling_dialog), NULL);

    GTK_WIDGET_SET_FLAGS(mf_deletebtn, GTK_CAN_DEFAULT);
    GTK_WIDGET_SET_FLAGS(mf_closebtn, GTK_CAN_DEFAULT);

    GLADE_HOOKUP_OBJECT_NO_REF(mf_dialog, mf_dialog, "mf_dialog");
    GLADE_HOOKUP_OBJECT_NO_REF(mf_dialog, dialog_vbox, "vbox");
    GLADE_HOOKUP_OBJECT(mf_dialog, folder_view, "folder_view");

    return mf_dialog;
}

GtkWidget *create_change_folder_dialog(GtkWindow *parent)
{
    GtkWidget *cf_dialog;
    GtkWidget *dialog_vbox;
    GtkWidget *dialog_action_area;
    GtkWidget *cf_okbtn;
    GtkWidget *cf_newbtn;
    GtkWidget *cf_cancelbtn;
    GtkWidget *folder_view;
    GtkWidget *folder_scrollwindow;

    //g_assert(app_data != NULL);
    //g_assert(app_data->app_ui_data != NULL);
    
    cf_dialog = gtk_dialog_new();
    gtk_window_set_title(GTK_WINDOW(cf_dialog), _("rss_ti_change_folder"));
//     gtk_window_set_position(GTK_WINDOW(cf_dialog), GTK_WIN_POS_CENTER);
    gtk_window_set_default_size(GTK_WINDOW(cf_dialog), CHANGE_FOLDER_DIALOG_XY_SIZE,
                                CHANGE_FOLDER_DIALOG_XY_SIZE);
    gtk_window_set_type_hint(GTK_WINDOW(cf_dialog),
                 GDK_WINDOW_TYPE_HINT_DIALOG);
    gtk_window_set_transient_for(GTK_WINDOW(cf_dialog), parent);
    gtk_window_set_modal(GTK_WINDOW(cf_dialog), TRUE);
    gtk_dialog_set_has_separator(GTK_DIALOG(cf_dialog), FALSE);

    dialog_vbox = GTK_DIALOG(cf_dialog)->vbox;
    gtk_widget_show(dialog_vbox);

    folder_scrollwindow = gtk_scrolled_window_new(NULL, NULL);
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW
                   (folder_scrollwindow), GTK_POLICY_NEVER,
                   GTK_POLICY_AUTOMATIC);
    gtk_widget_show(folder_scrollwindow);

    folder_view = gtk_tree_view_new();
    gtk_widget_show(folder_view);
    gtk_container_add(GTK_CONTAINER(folder_scrollwindow), folder_view);
    gtk_container_add(GTK_CONTAINER(dialog_vbox), folder_scrollwindow);

    gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(folder_view), FALSE);
    gtk_tree_view_set_reorderable(GTK_TREE_VIEW(folder_view), TRUE);


    dialog_action_area = GTK_DIALOG(cf_dialog)->action_area;
    gtk_widget_show(dialog_action_area);
    gtk_button_box_set_layout(GTK_BUTTON_BOX(dialog_action_area),
                  GTK_BUTTONBOX_END);

    cf_okbtn = gtk_button_new_with_label(_("rss_bd_ok"));
    gtk_widget_show(cf_okbtn);
    gtk_dialog_add_action_widget(GTK_DIALOG(cf_dialog), cf_okbtn,
                 GTK_RESPONSE_OK);

    cf_newbtn = gtk_button_new_with_label(_("rss_bd_new"));
    gtk_widget_show(cf_newbtn);
    gtk_dialog_add_action_widget(GTK_DIALOG(cf_dialog), cf_newbtn,
                 RSS_RESPONSE_NEW_FOLDER);

    cf_cancelbtn = gtk_button_new_with_label(_("rss_bd_cancel"));
    gtk_widget_show(cf_cancelbtn);
    gtk_dialog_add_action_widget(GTK_DIALOG(cf_dialog), cf_cancelbtn,
                 GTK_RESPONSE_CANCEL);

    GLADE_HOOKUP_OBJECT_NO_REF(cf_dialog, cf_dialog, "cf_dialog");
    GLADE_HOOKUP_OBJECT_NO_REF(cf_dialog, dialog_vbox, "vbox");
    GLADE_HOOKUP_OBJECT(cf_dialog, folder_view, "folder_view");

    return cf_dialog;
}


GtkWidget *create_prefdialog(void)
{
    GtkWidget *prefdialog;
    GtkWidget *dialog_vbox;
    GtkWidget *dialog_action_area;
    GtkWidget *prefokbtn;
    GtkWidget *prefcancelbtn;
    GtkWidget *openLinksLabel;
    GtkWidget *sortOrderLabel;
    GtkWidget *updateFeedsLabel;
    GtkWidget *openLinksPulldown;
    GtkWidget *sortOrderPulldown;
    GtkWidget *interval_label;
    GtkWidget *interval_editor;
    GtkWidget *radio_button_manually;
    GtkWidget *radio_button_automatically;
    GtkWidget *prefvbox;
    GtkWidget *table2;//, *table;
   
    //g_assert(app_data != NULL);
    //g_assert(app_data->app_ui_data != NULL);
    
    gchar     *str = NULL;
    //gchar     *saved_locale = NULL;

    GtkSizeGroup *group =
    GTK_SIZE_GROUP(gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL));

    prefdialog = gtk_dialog_new();
    gtk_window_set_title(GTK_WINDOW(prefdialog), _("rss_ti_settings"));
//     gtk_window_set_position(GTK_WINDOW(prefdialog), GTK_WIN_POS_CENTER);
    gtk_window_set_default_size(GTK_WINDOW(prefdialog), PREF_DIALOG_X_SIZE, -1);
    gtk_window_set_type_hint(GTK_WINDOW(prefdialog),
                 GDK_WINDOW_TYPE_HINT_DIALOG);
    gtk_window_set_transient_for(GTK_WINDOW(prefdialog),
                 GTK_WINDOW(app_data->app_ui_data->app));
    gtk_window_set_modal(GTK_WINDOW(prefdialog), TRUE);
    gtk_dialog_set_has_separator(GTK_DIALOG(prefdialog), FALSE);

    ossohelp_dialog_help_enable(
            GTK_DIALOG(prefdialog),
            RSS_NEWS_READER_HELP_SETTINGS,
            app_data->osso);

    dialog_vbox = GTK_DIALOG(prefdialog)->vbox;

    gtk_container_set_border_width(GTK_CONTAINER(dialog_vbox),
                        HILDON_MARGIN_DEFAULT);

    // vbox 
    prefvbox = gtk_vbox_new (FALSE,0);
    gtk_widget_show (prefvbox);
         
    gtk_container_add(GTK_CONTAINER(dialog_vbox), prefvbox);
    //Open Links in
    openLinksPulldown = gtk_combo_box_new_text();    

    gtk_combo_box_append_text(GTK_COMBO_BOX(openLinksPulldown),
                  _("rss_ia_reuse_window"));
    gtk_combo_box_append_text(GTK_COMBO_BOX(openLinksPulldown),
                  _("rss_ia_new_window"));

    gtk_combo_box_set_active(GTK_COMBO_BOX(openLinksPulldown), 0);
    gtk_widget_show(GTK_WIDGET(openLinksPulldown));

    openLinksLabel = hildon_caption_new(group, _("rss_ia_open_links_in"),
                    openLinksPulldown, NULL,
                    HILDON_CAPTION_OPTIONAL);
    gtk_widget_show(openLinksLabel);
    gtk_box_pack_start (GTK_BOX (prefvbox), openLinksLabel, FALSE, FALSE, 0);
    
    //Sort order    
    sortOrderPulldown = gtk_combo_box_new_text();
    gtk_combo_box_append_text(GTK_COMBO_BOX(sortOrderPulldown),
                  _("rss_ia_newest_first"));
    gtk_combo_box_append_text(GTK_COMBO_BOX(sortOrderPulldown),
                  _("rss_ia_oldest_first"));
    gtk_combo_box_set_active(GTK_COMBO_BOX(sortOrderPulldown), 0);
    gtk_widget_show(GTK_WIDGET(sortOrderPulldown));

    sortOrderLabel = hildon_caption_new(group, _("rss_ia_sort_order"),
                    sortOrderPulldown, NULL,
                    HILDON_CAPTION_OPTIONAL);
    gtk_widget_show(sortOrderLabel);
    gtk_box_pack_start (GTK_BOX (prefvbox), sortOrderLabel, FALSE, FALSE, 0);


    //Refresh feeds. We need a table for the right handside: table
   
    // the 2 radio buttons in 1 table table1
    radio_button_manually = gtk_radio_button_new(NULL);
    g_signal_connect(radio_button_manually, "toggled",
             radio_button_manually_toggled_cb, (gpointer) prefdialog);
    gtk_widget_show(radio_button_manually);
    GSList *rbgroup =
        gtk_radio_button_get_group(GTK_RADIO_BUTTON
                   (radio_button_manually));
    radio_button_automatically =
       gtk_radio_button_new(rbgroup);
    gtk_widget_show(radio_button_automatically);

    GtkWidget *ghost = gtk_hbox_new(FALSE,0);
    gtk_widget_show(ghost);
    gtk_widget_set_usize(GTK_WIDGET(ghost),0,32);
                        
    GtkWidget *table1 = gtk_table_new(2,2,FALSE);
    
    gtk_table_attach (GTK_TABLE(table1), radio_button_manually, 0, 1, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
    gtk_table_attach (GTK_TABLE(table1), radio_button_automatically, 0, 1, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
    gtk_table_attach (GTK_TABLE(table1), ghost, 0, 1, 2, 3, GTK_FILL, GTK_FILL, 0, 0);
    gtk_widget_show(table1);            
    
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio_button_manually),
                 TRUE);

    // three rows : Only manually/Automatically/Time-editor in table2
    table2 = gtk_table_new(1,3,FALSE);
    
    gtk_widget_show (table2); 
    
    GtkWidget *rb_manual = gtk_label_new(_("rss_ia_refresh_only_manually"));
    gtk_misc_set_alignment(GTK_MISC(rb_manual),0.0,0.5);
    gtk_widget_show(rb_manual);
    
    GtkWidget *rb_automatic = gtk_label_new(_("rss_ia_refresh_automatically"));                                 
    gtk_misc_set_alignment(GTK_MISC(rb_automatic),0.0,0.5);
    gtk_widget_show(rb_automatic);

    //time editor   
    /* TODO: This is an ugly hack to make it possible 
    to view the time picker in 24h mode */
    //saved_locale = save_and_change_time_locale ( "fi_FI" );

    interval_editor = hildon_time_editor_new();
    gtk_widget_show(interval_editor);
    hildon_time_editor_enable_duration_mode(HILDON_TIME_EDITOR
                        (interval_editor), TRUE);
    hildon_time_editor_show_seconds(HILDON_TIME_EDITOR(interval_editor),
                    FALSE);
    hildon_time_editor_set_duration_range(HILDON_TIME_EDITOR
                      (interval_editor), MIN_INTERVAL * 60,
                      24 * 60 * 60);
    /*       
    interval_label = hildon_caption_new(NULL, _("rss_ia_interval"),
                    interval_editor, NULL,
                    HILDON_CAPTION_OPTIONAL);
    */
    str = create_label_with_colon(_("rss_ia_interval"));
    interval_label = gtk_label_new(str);
    g_free(str);
    gtk_misc_set_alignment(GTK_MISC(interval_label),0.0,0.5);
    gtk_widget_set_usize(GTK_WIDGET(interval_label),325,10);
    gtk_widget_show(interval_label);
   
    gtk_table_attach (GTK_TABLE(table2), rb_manual, 0, 1, 0, 1, GTK_FILL, GTK_FILL, 10, 2);
    gtk_table_attach (GTK_TABLE(table2), rb_automatic, 0, 1, 1, 2, GTK_FILL, GTK_FILL, 10, 2);
    gtk_table_attach (GTK_TABLE(table2), interval_label, 0, 1, 2, 3, GTK_FILL, GTK_FILL, 10, 2);
    gtk_table_attach (GTK_TABLE(table2), interval_editor , 0, 1, 2, 3, GTK_FILL, GTK_FILL, 10, 0);                       
    
    //the box for radio buttons and text/time-editor : radiobox_table
    GtkWidget *radiobox_table = gtk_table_new(2,1,FALSE);
    gtk_table_attach (GTK_TABLE(radiobox_table), table1, 0, 1, 0, 1, 0, 0, 0, 0);
    gtk_table_attach (GTK_TABLE(radiobox_table), table2, 1, 2, 0, 1, 0, 0, 0, 0);
    gtk_widget_show(radiobox_table);
    
    //gtk_table_attach (GTK_TABLE(table), radiobox_table, 1, 2, 2, 5, 0, 0, 0, 0);
            
    //pack it in
    updateFeedsLabel = hildon_caption_new(group, _("rss_ia_refresh_feeds"),
                      radiobox_table, NULL,
                      HILDON_CAPTION_OPTIONAL);
    hildon_caption_set_label_alignment(updateFeedsLabel,0);    
    gtk_widget_show(updateFeedsLabel);
    gtk_box_pack_start (GTK_BOX (prefvbox), updateFeedsLabel, FALSE, FALSE, 0);

    //action area for buttons    
    dialog_action_area = GTK_DIALOG(prefdialog)->action_area;
    gtk_widget_show(dialog_action_area);
    gtk_button_box_set_layout(GTK_BUTTON_BOX(dialog_action_area),
                  GTK_BUTTONBOX_END);

    prefokbtn = gtk_button_new_with_label(_("rss_bd_ok"));
    gtk_widget_show(prefokbtn);
    gtk_dialog_add_action_widget(GTK_DIALOG(prefdialog), prefokbtn,
                 GTK_RESPONSE_OK);

    prefcancelbtn = gtk_button_new_with_label(_("rss_bd_cancel"));
    gtk_widget_show(prefcancelbtn);
    gtk_dialog_add_action_widget(GTK_DIALOG(prefdialog), prefcancelbtn,
                 GTK_RESPONSE_CANCEL);

    GTK_WIDGET_SET_FLAGS(prefokbtn, GTK_CAN_DEFAULT);

    g_signal_connect(G_OBJECT(prefdialog), "response",
             G_CALLBACK(on_prefdialog_response), NULL /*saved_locale*/);
    g_signal_connect(G_OBJECT(prefdialog), "key-press-event",
             G_CALLBACK(key_press_for_cancelling_dialog), NULL);
    
    GLADE_HOOKUP_OBJECT_NO_REF(prefdialog, prefdialog, "prefdialog");
    GLADE_HOOKUP_OBJECT_NO_REF(prefdialog, dialog_vbox, "dialog_vbox");

    GLADE_HOOKUP_OBJECT(prefdialog, openLinksPulldown,
            "openLinksPulldown");
    GLADE_HOOKUP_OBJECT(prefdialog, sortOrderPulldown,
            "sortOrderPulldown");
    GLADE_HOOKUP_OBJECT(prefdialog, radio_button_manually,
            "radio_button_manually");
    GLADE_HOOKUP_OBJECT(prefdialog, radio_button_automatically,
            "radio_button_automatically");
    GLADE_HOOKUP_OBJECT(prefdialog, interval_editor,
            "update_interval_widget");

    return prefdialog;
}

GtkWidget *create_newfolderdialog(folderPtr folder,
                  GtkTreeView * parent_treeview, GtkWindow *parent)
{
    GtkWidget *newfolderdialog;
    GtkWidget *dialog_vbox6;
    GtkWidget *vbox10;
    GtkWidget *label20;
    GtkWidget *foldertitleentry;
    GtkWidget *dialog_action_area6;
    GtkWidget *button3;
    GtkWidget *button2;
    GtkWidget *labelFolder;
    GtkWidget *location;
    GtkWidget *hboxLocation;
    GtkWidget *folderIcon;
    GtkWidget *alignment;
    int length = 0;

    //g_assert(app_data != NULL);
    //g_assert(app_data->app_ui_data != NULL);
    
    GtkSizeGroup *group =
    GTK_SIZE_GROUP(gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL));

    newfolderdialog = gtk_dialog_new();

    /* Set parent_folder so that we can create the new folder to
       correct place.
     */
    //g_assert(newfolderdialog != NULL);
    g_object_set_data(G_OBJECT(newfolderdialog), "parent_folder", folder);
    g_object_set_data(G_OBJECT(newfolderdialog), "folder_treeview",
              parent_treeview);
    g_object_set_data(G_OBJECT(newfolderdialog), "folder_treemodel",
              gtk_tree_view_get_model(parent_treeview));


    gtk_window_set_title(GTK_WINDOW(newfolderdialog),
             _("rss_ti_new_folder"));
//     gtk_window_set_position(GTK_WINDOW(newfolderdialog), GTK_WIN_POS_CENTER);
    gtk_window_set_type_hint(GTK_WINDOW(newfolderdialog),
                 GDK_WINDOW_TYPE_HINT_DIALOG);
    gtk_window_set_transient_for(GTK_WINDOW(newfolderdialog), parent);
    gtk_window_set_modal(GTK_WINDOW(newfolderdialog), TRUE);
    gtk_dialog_set_has_separator(GTK_DIALOG(newfolderdialog), FALSE);

    ossohelp_dialog_help_enable(
            GTK_DIALOG(newfolderdialog),
            RSS_NEWS_READER_HELP_MANAGE,
            app_data->osso);

    dialog_vbox6 = GTK_DIALOG(newfolderdialog)->vbox;
    gtk_widget_show(dialog_vbox6);

    vbox10 = gtk_vbox_new(FALSE, 0);
    gtk_widget_show(vbox10);
    gtk_box_pack_start(GTK_BOX(dialog_vbox6), vbox10, TRUE, TRUE, 0);

    foldertitleentry = gtk_entry_new();
    g_object_set(G_OBJECT(foldertitleentry), "autocap", FALSE, NULL);
    gtk_widget_show(foldertitleentry);
    gtk_entry_set_activates_default(GTK_ENTRY(foldertitleentry), TRUE);

    label20 = hildon_caption_new(group, _("rss_ia_name"),
                 foldertitleentry, NULL,
                 HILDON_CAPTION_OPTIONAL);
    gtk_widget_show(label20);
    gtk_box_pack_start(GTK_BOX(vbox10), label20, FALSE, FALSE, 0);

    if (folder != NULL) {
    if (folder->title != NULL) {
        location = gtk_label_new(folder->title);
    } else {
        /* This is probably an error. */
        location = gtk_label_new("");
    }
    } else {
    location = gtk_label_new(_("rss_ia_news_feed_folders"));
    }
    length = g_utf8_strlen(_("rss_ia_news_feed_folders"), -1);
    if(length > LABEL_MAX_WIDTH)
         gtk_label_set_max_width_chars(GTK_LABEL(location), length);
    else
         gtk_label_set_max_width_chars(GTK_LABEL(location), LABEL_MAX_WIDTH);
    gtk_label_set_ellipsize(GTK_LABEL(location), PANGO_ELLIPSIZE_END);
    gtk_widget_show(location);

    folderIcon = gtk_image_new_from_pixbuf(icons[ICON_FOLDER_CLOSED]);
    gtk_widget_show(folderIcon);

    hboxLocation = gtk_hbox_new(FALSE, HILDON_MARGIN_DEFAULT);
    gtk_widget_show(hboxLocation);
    gtk_box_pack_start_defaults(GTK_BOX(hboxLocation), folderIcon);
    gtk_box_pack_start_defaults(GTK_BOX(hboxLocation), location);

    alignment = gtk_alignment_new(0, 0.5, 0, 0);
    gtk_widget_show(alignment);
    gtk_container_add(GTK_CONTAINER(alignment), hboxLocation);

    labelFolder = hildon_caption_new(group, _("rss_ia_location"),
                     alignment, NULL,
                     HILDON_CAPTION_OPTIONAL);
    gtk_widget_show(labelFolder);
    gtk_box_pack_start_defaults(GTK_BOX(vbox10), labelFolder);

    dialog_action_area6 = GTK_DIALOG(newfolderdialog)->action_area;
    gtk_widget_show(dialog_action_area6);
    gtk_button_box_set_layout(GTK_BUTTON_BOX(dialog_action_area6),
                  GTK_BUTTONBOX_END);

    button2 = gtk_button_new_with_label(_("rss_bd_ok"));
    gtk_widget_show(button2);
    gtk_dialog_add_action_widget(GTK_DIALOG(newfolderdialog), button2,
                 GTK_RESPONSE_OK);
    GTK_WIDGET_SET_FLAGS(button2, GTK_CAN_DEFAULT);

    button3 = gtk_button_new_with_label(_("rss_bd_cancel"));
    gtk_widget_show(button3);
    gtk_dialog_add_action_widget(GTK_DIALOG(newfolderdialog), button3,
                 GTK_RESPONSE_CANCEL);
    GTK_WIDGET_SET_FLAGS(button3, GTK_CAN_DEFAULT);

    g_signal_connect_swapped((gpointer) newfolderdialog, "delete_event",
                 G_CALLBACK(gtk_widget_hide),
                 GTK_OBJECT(newfolderdialog));
    g_signal_connect_swapped((gpointer) button3, "clicked",
                 G_CALLBACK(gtk_widget_hide),
                 GTK_OBJECT(newfolderdialog));
    g_signal_connect((gpointer) button2, "clicked",
             G_CALLBACK(on_newfolderbtn_clicked), newfolderdialog);
    /*g_signal_connect_data((gpointer) button2, "clicked",
              G_CALLBACK(gtk_widget_hide),
              GTK_OBJECT(newfolderdialog),
              NULL, G_CONNECT_AFTER | G_CONNECT_SWAPPED);*/

    /* Store pointers to all widgets, for use by lookup_widget(). */
    GLADE_HOOKUP_OBJECT_NO_REF(newfolderdialog, newfolderdialog,
                   "newfolderdialog");
    GLADE_HOOKUP_OBJECT_NO_REF(newfolderdialog, dialog_vbox6,
                   "dialog_vbox6");
    GLADE_HOOKUP_OBJECT(newfolderdialog, vbox10, "vbox10");
    GLADE_HOOKUP_OBJECT(newfolderdialog, label20, "label20");
    GLADE_HOOKUP_OBJECT(newfolderdialog, foldertitleentry,
            "foldertitleentry");
    GLADE_HOOKUP_OBJECT_NO_REF(newfolderdialog, dialog_action_area6,
                   "dialog_action_area6");
    GLADE_HOOKUP_OBJECT(newfolderdialog, button3, "button3");
    GLADE_HOOKUP_OBJECT(newfolderdialog, button2, "button2");

    gtk_widget_grab_default(button2);
    return newfolderdialog;
}

GtkWidget *create_foldernamedialog(void)
{
    GtkWidget *foldernamedialog;
    GtkWidget *dialog_vbox7;
    GtkWidget *foldernameentry;
    GtkWidget *foldernamecaption;
    GtkWidget *dialog_action_area7;
    GtkWidget *cancelbutton1;
    GtkWidget *foldernamechangebtn;

    //g_assert(app_data != NULL);
    //g_assert(app_data->app_ui_data != NULL);
    
    foldernamedialog = gtk_dialog_new();
    gtk_window_set_title(GTK_WINDOW(foldernamedialog),
             _("rss_ti_rename_folder"));
//     gtk_window_set_position(GTK_WINDOW(foldernamedialog), GTK_WIN_POS_CENTER);
    gtk_window_set_type_hint(GTK_WINDOW(foldernamedialog),
                 GDK_WINDOW_TYPE_HINT_DIALOG);
    gtk_window_set_transient_for(GTK_WINDOW(foldernamedialog),
                 GTK_WINDOW(app_data->app_ui_data->app));
    gtk_window_set_modal(GTK_WINDOW(foldernamedialog), TRUE);
    gtk_dialog_set_has_separator(GTK_DIALOG(foldernamedialog), FALSE);

    ossohelp_dialog_help_enable(
            GTK_DIALOG(foldernamedialog),
            RSS_NEWS_READER_HELP_MANAGE,
            app_data->osso);

    dialog_vbox7 = GTK_DIALOG(foldernamedialog)->vbox;
    gtk_widget_show(dialog_vbox7);

    foldernameentry = gtk_entry_new();
    g_object_set(G_OBJECT(foldernameentry), "autocap", FALSE, NULL);
    gtk_widget_show(foldernameentry);
    foldernamecaption = hildon_caption_new(NULL, _("rss_ia_name"),
                       foldernameentry, NULL,
                       HILDON_CAPTION_OPTIONAL);
    gtk_widget_show(foldernamecaption);
    gtk_box_pack_start(GTK_BOX(dialog_vbox7), foldernamecaption, FALSE,
               FALSE, 12);
    gtk_entry_set_activates_default(GTK_ENTRY(foldernameentry), TRUE);

    dialog_action_area7 = GTK_DIALOG(foldernamedialog)->action_area;
    gtk_widget_show(dialog_action_area7);
    gtk_button_box_set_layout(GTK_BUTTON_BOX(dialog_action_area7),
                  GTK_BUTTONBOX_END);

    foldernamechangebtn = gtk_button_new_with_label(_("rss_bd_ok"));
    gtk_widget_show(foldernamechangebtn);
    gtk_dialog_add_action_widget(GTK_DIALOG(foldernamedialog),
                 foldernamechangebtn, 0);
    GTK_WIDGET_SET_FLAGS(foldernamechangebtn, GTK_CAN_DEFAULT);

    cancelbutton1 = gtk_button_new_with_label(_("rss_bd_cancel"));
    gtk_widget_show(cancelbutton1);
    gtk_dialog_add_action_widget(GTK_DIALOG(foldernamedialog),
                 cancelbutton1, 0);
    GTK_WIDGET_SET_FLAGS(cancelbutton1, GTK_CAN_DEFAULT);

    g_signal_connect_swapped((gpointer) foldernamedialog, "delete_event",
                 G_CALLBACK(gtk_widget_hide),
                 GTK_OBJECT(foldernamedialog));
    g_signal_connect_swapped((gpointer) cancelbutton1, "clicked",
                 G_CALLBACK(gtk_widget_hide),
                 GTK_OBJECT(foldernamedialog));
    g_signal_connect((gpointer) foldernamechangebtn, "clicked",
             G_CALLBACK(on_foldernamechangebtn_clicked), NULL);

    /* Store pointers to all widgets, for use by lookup_widget(). */
    GLADE_HOOKUP_OBJECT_NO_REF(foldernamedialog, foldernamedialog,
                   "foldernamedialog");
    GLADE_HOOKUP_OBJECT_NO_REF(foldernamedialog, dialog_vbox7,
                   "dialog_vbox7");
    GLADE_HOOKUP_OBJECT(foldernamedialog, foldernameentry,
            "foldernameentry");
    GLADE_HOOKUP_OBJECT_NO_REF(foldernamedialog, dialog_action_area7,
                   "dialog_action_area7");
    GLADE_HOOKUP_OBJECT(foldernamedialog, cancelbutton1, "cancelbutton1");
    GLADE_HOOKUP_OBJECT(foldernamedialog, foldernamechangebtn,
            "foldernamechangebtn");

    gtk_widget_grab_default(foldernamechangebtn);
    return foldernamedialog;
}

/* 2006May31: Gutter position stuff. It needs to be saved */
gint get_gutter_position(void)
{
    //ULOG_DEBUG("get_gutter_position: = %d", gtk_paned_get_position(GTK_PANED(newsbox)));
    return gtk_paned_get_position(GTK_PANED(newsbox));
}
void set_gutter_position(gint pos)
{
    gtk_paned_set_position(GTK_PANED(newsbox), pos);
    

}
