/* 
 * gzip/zlib uncompression, Liferea reuses a stripped
 * version of the SnowNews code...
 *
 * Snownews - A lightweight console RSS newsreader
 * 
 * Copyright 2003-2004 Oliver Feiler <kiza@kcore.de> and
 *                     Rene Puls <rpuls@gmx.net>
 *
 * conversions.c
 *
 * Please read the file README.patching before changing any code in this file!
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

/* For strsep */
#define _BSD_SOURCE 1

#include <string.h>
#include <iconv.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <zlib.h>
#include <osso-log.h>

#include "compat.h"
#include "conversions.h"

extern struct entity *first_entity;

/************************************************************************/
/*                        PUBLIC FUNCTIONS                              */
/************************************************************************/

/* Decompresses deflate compressed data. Probably unneeded,
 * see gzip_uncompress below.
 */
void *zlib_uncompress(void *in_buf, int in_size, int *out_size,
              int voodoo_magic)
{
    char tmpstring[1024];
    z_stream stream;
    char *out_buf = NULL;
    int out_buf_bytes = 0;
    char tmp_buf[512];
    int result;
    int new_bytes;

    /* Prepare the stream structure. */
    stream.zalloc = NULL;
    stream.zfree = NULL;
    stream.opaque = NULL;
    stream.next_in = in_buf;
    stream.avail_in = in_size;
    stream.next_out = tmp_buf; /* CHECK does this and the next parameter need to be initialized
                                  and if so, is this done correctly? */
    stream.avail_out = 512; /*sizeof tmp_buf;*/ /* CHECK */

    if (out_size != NULL)
    *out_size = 0;

    /* Deflated data from GZIP files can only be decompressed with voodoo magic(tm)!
       CHECK: does this cause inflate function to return Z_DATA_ERROR? */
    ULOG_DEBUG("Mysterious MAX_WBITS: %d",MAX_WBITS);
    if (voodoo_magic)
    {
    ULOG_DEBUG("inflateInit2 returned: %d",inflateInit2(&stream, -MAX_WBITS));
    }
    else
    inflateInit(&stream);

    do {
    /* Should be Z_FINISH? */
    result = inflate(&stream, /*Z_SYNC_FLUSH*/Z_NO_FLUSH);
    switch (result) {
    case Z_ERRNO:
    case Z_NEED_DICT:
    case Z_BUF_ERROR:
    case Z_MEM_ERROR:
    case Z_DATA_ERROR:
    case Z_VERSION_ERROR:
        inflateEnd(&stream);
        free(out_buf);
        snprintf(tmpstring, sizeof(tmpstring),
             "Errors occurred while parsing this feed"/*, result,
             stream.msg*/);
        ULOG_DEBUG(tmpstring, 2);
        ULOG_DEBUG("ZLIB: Errors. Result: %d",result);
        return NULL;
    }
    if (stream.avail_out < 512/*sizeof tmp_buf*/) {
        /* Add the new uncompressed data to our output buffer. */
        new_bytes = 512/*sizeof tmp_buf*/ - stream.avail_out;
        out_buf = realloc(out_buf, out_buf_bytes + new_bytes);
        memcpy(out_buf + out_buf_bytes, tmp_buf, new_bytes);
        out_buf_bytes += new_bytes;
        stream.next_out = tmp_buf;
        stream.avail_out = 512/*sizeof tmp_buf*/;
    } else {
        /* For some reason, inflate() didn't write out a single byte. */
        inflateEnd(&stream);
        free(out_buf);
        ULOG_DEBUG("ZLIB: No output during decompression");
        return NULL;
    }
    } while (result != Z_STREAM_END);

    inflateEnd(&stream);

    /* Null-terminate the output buffer so it can be handled like a string. */
    out_buf = realloc(out_buf, out_buf_bytes + 1);
    out_buf[out_buf_bytes] = 0;

    /* The returned size does NOT include the additionall null byte! */
    if (out_size != NULL)
    *out_size = out_buf_bytes;
 
    ULOG_DEBUG("ZLIB OK: out_buf: %d",(out_buf==NULL?0:1));

    return out_buf;
}

/* Decompressed gzip,deflate compressed data. This is what the webservers
 * usually send. CHECK if the header is processed with zlib */
void *gzip_uncompress(void *in_buf, int in_size, int *out_size)
{
    char tmpstring[1024];
    struct gzip_header *header;
    char *data_start;
    int offset = 10; /*sizeof *header;*/ /* CHECK */

    ULOG_DEBUG("header size: %d",sizeof *header);
    
    header = in_buf;

    if (out_size != NULL)
    *out_size = 0;

    if ((header->magic[0] != 0x1F) || (header->magic[1] != 0x8B)) {
        ULOG_DEBUG("GZIP: Invalid magic bytes for GZIP data");
    return NULL;
    }

    if (header->method != 8) {
        ULOG_DEBUG("GZIP: rss_uncompress_deflate");
    return NULL;
    }

    ULOG_DEBUG("GZIP: header->flags %d",header->flags);    
    if (header->flags != 0 && header->flags != 8) {
        ULOG_DEBUG("GZIP: Errors occured while parsing this feed");
    snprintf(tmpstring, sizeof(tmpstring),
         "Errors occurred while parsing this feed"/*, header->flags*/);
    ULOG_DEBUG(tmpstring, 2);
    return NULL;
    }

    if (header->flags & 8) {
    /* skip the file name */
    while (offset < in_size) {
        if (((char *) in_buf)[offset] == 0) {
        offset++;
        break;
        }
        offset++;
    }
    }

    data_start = (char *) in_buf + offset;

    /* CHECK that data really starts at the right location */
    ULOG_DEBUG("in_buf: %p data starts at: %p",in_buf,data_start);

    return zlib_uncompress(data_start, in_size - offset - 8, out_size, 1);
}


char *base64encode(char const *inbuf, unsigned int inbuf_size)
{
    static unsigned char const alphabet[64] =
    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";

    char *outbuf = NULL;
    unsigned int inbuf_pos = 0;
    unsigned int outbuf_pos = 0;
    unsigned int outbuf_size = 0;
    int bits = 0;
    int char_count = 0;

    outbuf = malloc(1);

    while (inbuf_pos < inbuf_size) {

    bits |= *inbuf;
    char_count++;

    if (char_count == 3) {
        outbuf = realloc(outbuf, outbuf_size + 4);
        outbuf_size += 4;
        outbuf[outbuf_pos + 0] = alphabet[bits >> 18];
        outbuf[outbuf_pos + 1] = alphabet[(bits >> 12) & 0x3f];
        outbuf[outbuf_pos + 2] = alphabet[(bits >> 6) & 0x3f];
        outbuf[outbuf_pos + 3] = alphabet[bits & 0x3f];
        outbuf_pos += 4;
        bits = 0;
        char_count = 0;
    }

    inbuf++;
    inbuf_pos++;
    bits <<= 8;
    }

    if (char_count > 0) {
    bits <<= 16 - (8 * char_count);
    outbuf = realloc(outbuf, outbuf_size + 4);
    outbuf_size += 4;
    outbuf[outbuf_pos + 0] = alphabet[bits >> 18];
    outbuf[outbuf_pos + 1] = alphabet[(bits >> 12) & 0x3f];
    if (char_count == 1) {
        outbuf[outbuf_pos + 2] = '=';
        outbuf[outbuf_pos + 3] = '=';
    } else {
        outbuf[outbuf_pos + 2] = alphabet[(bits >> 6) & 0x3f];
        outbuf[outbuf_pos + 3] = '=';
    }
    outbuf_pos += 4;
    }

    outbuf = realloc(outbuf, outbuf_size + 1);
    outbuf[outbuf_pos] = 0;

    return outbuf;
}


/* Returns NULL on invalid input */
char* decodechunked(char * chunked, unsigned int *inputlen) {
    /* We can reuse the same buffer to dechunkify it:
     * the data size will never increase. */
    char *orig = chunked, *dest = chunked;
    unsigned long chunklen;
    while((chunklen = strtoul(orig, &orig, 16))) {
        /* process one more chunk: */
        /* skip chunk-extension part */
        while(*orig && (*orig != '\r'))
            orig++;
        /* skip '\r\n' after chunk length */
        orig += 2;
        if(( chunklen > (chunked + *inputlen - orig)))
            /* insane chunk length. Well... */
            return NULL;
        memmove(dest, orig, chunklen);
        dest += chunklen;
        orig += chunklen;
        /* and go to the next chunk */
    }
    *dest = '\0';
    *inputlen = dest - chunked;
    
    return chunked;
}
