/**
 * @file ui_mainwindow.c some functions concerning the main window 
 *
 * Copyright (C) 2004 Nathan J. Conrad <t98502@users.sourceforge.net>
 * Copyright (C) 2004 Lars Lindner <lars.lindner@gmx.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 * 
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <gdk/gdkkeysyms.h>
#include <hildon-widgets/hildon-defines.h>
#include <hildon-widgets/gtk-infoprint.h>
#include <hildon-widgets/hildon-note.h>
#include <osso-log.h>
#include <langinfo.h>
#include "interface.h"
#include "support.h"
#include "conf.h"
#include "debug.h"
#include "callbacks.h"
#include "ui_feedlist.h"
#include "ui_mainwindow.h"
#include "ui_folder.h"
#include "ui_itemlist.h"
#include "ui_session.h"
#include "ui_queue.h"
#include "update.h"
#include "htmlview.h"
#include "osso-helplib.h"
#include "dbus.h"
#include "textview.h"
#include "gtkhtml/gtkhtml_plugin.h"

/* The Hildon AppView for the application */
GtkWidget *mainwindow;

/* A container for holding the feed list container and the search view */
GtkWidget *newsbox;

/* A GtkTreeView widget for displaying the feed list */
GtkWidget *feedlist;

/* A container for the feed list, used to show or hide the folder view */
GtkWidget *feedlist_container;

/* a GtkTextView widget for displaying html output on the right side of the
   user interface */
//static GtkWidget *htmlview = NULL;
//extern GtkWidget * gtkhtml_htmlview;
/* Application specific date */

extern AppData *app_data;

/* When the search is in progress, this variable is updated to indicate the
   number of search entries found */
//extern guint rss_search_found;

/* The number of unread posts */
gulong rss_unread;

/* The last refresh time of the news reader */
time_t rss_updated;

static GtkWidget *search_count = NULL;
gfloat zoom;

gboolean itemlist_mode = FALSE;    /* TRUE means three pane, FALSE means two panes */

/************************************************************************/
/*                        PUBLIC FUNCTIONS                              */
/************************************************************************/

GtkWidget *ui_mainwindow_get_active_htmlview()
{
 //   return htmlview;
 return app_data->app_ui_data->html_scrollpane;
}

extern htmlviewPluginInfo *htmlviewInfo;

void ui_mainwindow_set_search(gboolean search)
{
    //g_assert(NULL != search_count);
    g_return_if_fail(NULL != search_count);
    //g_assert(NULL != app_data->app_ui_data->html_scrollpane);
    g_return_if_fail(NULL != app_data->app_ui_data->html_scrollpane);

    if (TRUE == search) {
        //g_assert(app_data != NULL);
        //g_assert(app_data->app_ui_data != NULL);
    
        hildon_appview_set_title(app_data->app_ui_data->main_view,
                 _("rss_ti_search"));
        hildon_app_set_two_part_title(app_data->app_ui_data->app, TRUE);
        debug0(DEBUG_GUI, "Setting search-view on.");
        gtk_widget_show(search_count);
    } else {
    debug0(DEBUG_GUI, "Setting search-view off.");
    gtk_widget_hide(search_count);
    }
}

void ui_mainwindow_construct_search_view()
{
    gchar *tmp = NULL;
    gchar *str = NULL;
    //g_assert(app_data != NULL);
    g_return_if_fail(app_data != NULL);
    //g_assert(app_data->app_ui_data != NULL);
    g_return_if_fail(app_data->app_ui_data != NULL);
    
    if (displayed_node != NULL)
        if (displayed_node->type != FST_VFOLDER)
            return;

    ULOG_DEBUG("\n*******************************************\n\
                ui_mainwindow_construct_search_view\n\
                \n*******************************************\n ");
    if(app_data->app_ui_data->rss_search_found == 1)
        str = g_strdup_printf("<span weight=\"bold\">1 %s</span>",
                         _("rss_ia_item_found"));
    else
    {
        tmp = g_strdup_printf("<span weight=\"bold\">%s</span>",
                         _("rss_ia_items_found"));
    
        str = g_strdup_printf(tmp, app_data->app_ui_data->rss_search_found);
    }
    
    gtk_label_set_markup(GTK_LABEL
                  (app_data->app_ui_data->search_label), str);

    gtk_widget_show(app_data->app_ui_data->search_label);

    if(tmp != NULL)
        g_free(tmp);
    g_free(str);
}

void ui_mainwindow_set_mode(gboolean threePane)
{
    debug_enter("ui_mainwindow_set_mode");
    debug1(DEBUG_GUI, "Setting threePane mode: %s",
       threePane ? "on" : "off");

    //g_assert(NULL != newsbox);
    g_return_if_fail(NULL != newsbox);

    if (threePane == TRUE && (itemlist_mode == FALSE || app_data->app_ui_data->html_scrollpane == NULL)) {
    ULOG_ERR("Three-pane view is not supported!");

    } else if (threePane == FALSE
           && (itemlist_mode == TRUE || app_data->app_ui_data->html_scrollpane == NULL)) {
        gtk_widget_grab_focus(feedlist);
       /* Construct htmlview */
       if (app_data->app_ui_data->html_scrollpane != NULL)
           gtk_widget_destroy(app_data->app_ui_data->html_scrollpane);
       app_data->app_ui_data->html_scrollpane = NULL; 
       //ULOG_DEBUG("tvh ui_mainwindow: ui_htmlviw_new starting...");
       //gtkhtml_htmlview = ui_htmlview_new();//tvh:IMPORTANT
       app_data->app_ui_data->html_scrollpane = ui_htmlview_new();

       //ULOG_DEBUG("tvh ui_mainwindow: ui_htmlviw_new done");

        //g_assert(app_data != NULL);
        g_return_if_fail(app_data != NULL);
        //g_assert(app_data->app_ui_data != NULL);
        g_return_if_fail(app_data->app_ui_data != NULL);
       
       /* Construct search-view */
       app_data->app_ui_data->search_label = gtk_label_new(NULL);
       ui_mainwindow_construct_search_view();
    
       gtk_misc_set_alignment(GTK_MISC
                   (app_data->app_ui_data->search_label), 0,
                   0.5);
       search_count = gtk_vbox_new(FALSE, HILDON_MARGIN_DEFAULT);
       gtk_box_pack_start(GTK_BOX(search_count),
                  app_data->app_ui_data->search_label, FALSE,
               FALSE, 0);
       GtkWidget *search_view = gtk_vbox_new(FALSE, 0);
       gtk_box_pack_start(GTK_BOX(search_view), search_count, FALSE,
                  FALSE, 0);
       gtk_box_pack_end(GTK_BOX(search_view), app_data->app_ui_data->html_scrollpane, TRUE, TRUE, 0);
       gtk_widget_show_all(search_view);
       /*  GTkPaned now */
        //gtk_box_pack_end(GTK_BOX(newsbox), search_view, TRUE, TRUE, 0);
        gtk_paned_pack2 (GTK_PANED(newsbox), search_view, 
                                            TRUE, /*resize-able*/
                                            FALSE); /*shrink-able*/
       gtk_widget_show_all(newsbox);
    
       ui_mainwindow_set_search(FALSE);
        }


    itemlist_mode = threePane;

    debug_exit("ui_mainwindow_set_mode");
    //ULOG_DEBUG("tvh ui_mainwindow:  done");
}

void ui_mainwindow_finish(void)
{
    gchar *buffer = NULL;
    gchar *textbuffer = NULL;
    /*gchar *textformat = NULL;*/
    gchar *t1_raw = NULL;
    gchar *t1_format = NULL;
    gchar *t2_raw = NULL;
    gchar *t2_format = NULL;
    gchar *t3_raw = NULL;
    gchar *t3_format = NULL;
    gchar *t4_raw = NULL;
    gchar *t4_format = NULL;
    gint nr_feeds = 0;
    gint nr_unread = 0;
    gint nr_unread_feeds = 0;
    gint nr_marked = 0;
    glong last_poll_time = 0;
    gchar *update_time;
    gchar *update_date;

    //g_assert(app_data != NULL);
    g_return_if_fail(app_data != NULL);    
    g_assert(app_data->app_ui_data != NULL);
    g_return_if_fail(app_data->app_ui_data != NULL);    
    //ULOG_DEBUG("tvh ui_mainwindow_finish:  calling ui_feedlist_find_mainwindow_data......");    
    zoom = getNumericConfValue(LAST_ZOOMLEVEL) / 100.;

    ui_feedlist_find_mainwindow_data(NULL, &nr_feeds, &nr_unread,
                     &nr_unread_feeds, &nr_marked,
                     &last_poll_time);

    rss_unread = (long)nr_unread;
    
    ULOG_DEBUG("rss_updated: %ld last_poll_time: %ld", rss_updated, last_poll_time);
    
    /*if(rss_updated < (time_t)last_poll_time)
    {*/
         ULOG_DEBUG("Setting rss_updated to %ld", last_poll_time);
         rss_updated = (time_t)last_poll_time;
    /*}*/
    general_save();
    /* tvh: TODO commented this out due to bug 22893: Applet only gets signal when
        refreshing....FOLDER read doesn't affect applet. DOES IT?*/
    //rss_dbus_signal_applet(OSSO_RSS_FEED_READER_FOLDER_READ_SIGNAL);
    
    if (!app_data->app_ui_data->feed_displayed)
    {   
        /*last_poll_time = get_local_time_from_time_t(last_poll_time);*/
       update_time = get_localized_time(get_local_time_from_time_t(rss_updated));
       update_date = get_localized_date(get_local_time_from_time_t(rss_updated), TRUE);
       if(nr_feeds > 0)
       {
           t1_raw = g_strdup_printf("<p>%s\n</p>", _("rss_ia_you_have_subscribed_to_n_feeds"));
           t1_format = g_strdup_printf(t1_raw, nr_feeds);
           g_free(t1_raw);
       }
       else
           t1_format = g_strdup_printf("<p>%s\n</p>", _("rss_ia_no_feeds_subscribed"));
           
            if(rss_updated > 0)
       {
           t2_raw = g_strdup_printf("<p>%s\n</p>", _("rss_ia_last_refresh_on_date_at_time"));
           t2_format = g_strdup_printf(t2_raw, update_date, update_time);
           g_free(t2_raw);
       }        
            else
           t2_format = g_strdup_printf("<p>%s\n</p>", _("rss_ia_feeds_never_refreshed"));
           
       if(nr_unread > 0)
       {
           t3_raw = g_strdup_printf("<p>%s\n</p>", _("rss_ia_there_are_n_unread_posts"));
           t3_format = g_strdup_printf(t3_raw, nr_unread, nr_unread_feeds);
           g_free(t3_raw);
       }                
       else
           t3_format = g_strdup_printf("<p>%s\n</p>", _("rss_ia_no_unread"));
           
       if(nr_marked > 0)
       {
        t4_raw = g_strdup_printf("<p>%s\n</p>", _("rss_ia_you_have_saved_n_items"));
           t4_format = g_strdup_printf(t4_raw, nr_marked);
           g_free(t4_raw);
       }
       else
           t4_format = g_strdup_printf("<p>%s\n</p>", _("rss_ia_no_saved_posts"));
    
       textbuffer = g_strdup_printf("%s%s%s%s", t1_format, t2_format, t3_format, t4_format);
       
       addToHTMLBuffer(&buffer, textbuffer);
       
       //ui_htmlview_write(ui_mainwindow_get_active_htmlview(), buffer, NULL);
       //widget_alter_font_size(ui_htmlview_get_buffer(), DEFAULT_FONT_SIZE, TRUE, FALSE);

       //ULOG_DEBUG("tvh ui_mainwindow_finish:  writing general window now with textbuffer =%s",textbuffer);    
       
       ui_htmlview_write(app_data->app_ui_data->html_scrollpane, textbuffer, NULL, NULL);
        /* needed
        Otherwise, after a refresh_all action, ui_mainwindow_finish will be called
        if we don't set displayed_node to NULL, clicking on the previously selected feed wouldn't
        display it 
        */
       displayed_node = NULL;

        ULOG_DEBUG("ui_mainwindow_finish: getNumbericConfValue");
       zoom = getNumericConfValue(LAST_ZOOMLEVEL) / 100.;

       g_free(update_time);
       g_free(update_date);
       g_free(t1_format);
       g_free(t2_format);
       g_free(t3_format);
       g_free(t4_format);
       /*g_free(textformat);*/
       g_free(textbuffer);
       g_free(buffer);
    }

}

void on_unsubscribe_clicked(GtkButton * button, gpointer user_data)
{
    
    if (SFM_SEARCH == app_data->app_ui_data->search_mode){
        gtk_infoprint(GTK_WINDOW(app_data->app_ui_data->app), _("rss_ib_unable_unsubscribe_searching"));
        return;
    }

    /* NOTE: downloading images also uses SFM_DOWNLOAD mode */
    if ((SFM_DOWNLOAD == app_data->app_ui_data->search_mode) &&
        (get_counter(RSS_MAX) > 0 )){
        gtk_infoprint(GTK_WINDOW(app_data->app_ui_data->app), _("rss_ib_unable_unsubscribe"));
        return;
    }

    delete_selected_feed();
}

/*
 * Feed menu callbacks
 */

void on_menu_feed_new(GtkMenuItem * menuitem, gpointer user_data)
{
      if (!GTK_WIDGET_SENSITIVE(app_data->app_ui_data->menuitem_add)) {
        if (app_data->app_ui_data->search_mode == SFM_SEARCH)
            gtk_infoprint(GTK_WINDOW(app_data->app_ui_data->app),
                _("rss_ib_unable_add_feed_searching"));
        else if (app_data->app_ui_data->search_mode == SFM_DOWNLOAD)
            gtk_infoprint(GTK_WINDOW(app_data->app_ui_data->app),
                _("rss_ib_unable_add_feed"));
        else 
            ULOG_DEBUG("ooooooooooooooooooERRORooooooooooooooooooo");
        return;
      }
    on_newbtn_clicked(NULL, NULL);
}

void on_feed_insensitive(GtkMenuItem * menuitem, gpointer user_data)
{
    gtk_infoprint(GTK_WINDOW(app_data->app_ui_data->app), _("rss_ib_no_feed_selected"));
}

void on_menu_open(GtkMenuItem * menuitem, gpointer user_data)
{
    ULOG_DEBUG(__FUNCTION__);
    nodePtr ptr = (nodePtr) ui_feedlist_get_selected();
    if (ptr && ((FST_FEED == ptr->type) || (FST_VFOLDER == ptr->type))) {
    ui_feedlist_load_selected(ptr);
    } else {
    // Should be on_feed_insensitive, but here for safety.
    gtk_infoprint(GTK_WINDOW(app_data->app_ui_data->app), _("rss_ib_no_feed_selected"));
    }
}

gboolean delete_selected_feed()
{
    nodePtr ptr = (nodePtr) ui_feedlist_get_selected();

    if ((ptr != NULL) && (ptr->type != FST_FOLDER )) {
    return ui_feedlist_delete(ptr);
    } else {
    // Should be on_feed_insensitive, but here for safety.
    ULOG_WARN("Feed selection was NULL, or FOLDER cannot delete.");
    gtk_infoprint(GTK_WINDOW(app_data->app_ui_data->app), _("rss_ib_no_feed_selected"));
    }
    return FALSE;
}

void on_menu_delete(GtkMenuItem * menuitem, gpointer user_data)
{
      if (!GTK_WIDGET_SENSITIVE(app_data->app_ui_data->menuitem_add)) {
           if (app_data->app_ui_data->search_mode == SFM_SEARCH)
            gtk_infoprint(GTK_WINDOW(app_data->app_ui_data->app),
                _("rss_ib_unable_unsubscribe_searching"));
           else if (app_data->app_ui_data->search_mode == SFM_DOWNLOAD)    
            gtk_infoprint(GTK_WINDOW(app_data->app_ui_data->app),
                _("rss_ib_unable_unsubscribe"));
           
        return;
      }
    delete_selected_feed();
}


void on_mime_open_add_opml_requested(AppData * appdata, const gchar * opml_source)
{

    ULOG_DEBUG("on_menu_importopml .....\n");

    GtkWidget *importopmldialog;

    AppUIData *app_ui_data;
     
    //g_assert(app_data != NULL);
    g_return_if_fail(app_data != NULL);
    
    app_ui_data = app_data->app_ui_data;

    //g_assert(app_ui_data != NULL);
    g_return_if_fail(app_ui_data != NULL);
    /* create opml dialog with opml_source as the default source */
    importopmldialog = ui_feed_importopmldialog(GTK_WINDOW(app_ui_data->app), opml_source);
    if(importopmldialog != NULL)
        gtk_widget_show(importopmldialog);

}

/**
        
*/
void on_menu_importopml(GtkMenuItem * menuitem, gpointer user_data)
{
    (void) user_data;
    ULOG_DEBUG("on_menu_importopml .....\n");

    GtkWidget *importopmldialog;
    AppUIData *app_ui_data;
     
    //g_assert(app_data != NULL);
    g_return_if_fail(app_data != NULL);
    
    app_ui_data = app_data->app_ui_data;

    //g_assert(app_ui_data != NULL);
    g_return_if_fail(app_ui_data != NULL);
    if (!GTK_WIDGET_SENSITIVE(app_ui_data->menuitem_importopml)) {
        if (app_data->app_ui_data->search_mode == SFM_SEARCH)
            gtk_infoprint(GTK_WINDOW(app_data->app_ui_data->app),
                 _("rss_ib_unable_add_feed_searching"));        
        else if (app_data->app_ui_data->search_mode == SFM_DOWNLOAD)
            gtk_infoprint(GTK_WINDOW(app_data->app_ui_data->app),
                 _("rss_ib_unable_add_feed"));        
        else 
            ULOG_DEBUG("oooooooooooooooooooooooooERRORooooooooooooooooooooooooooooo");

        return;    
    }
    /* added low memory banner 28-June-06 */
    if (app_ui_data->low_mem)
    {
        gtk_infoprint( GTK_WINDOW(app_data->app_ui_data->app), 
                    dgettext("ke-recv", "memr_ib_operation_disabled") );
        return;                    
    }

    /*ui_feed.c*/
    /* create opml dialog with no default source */
    importopmldialog = ui_feed_importopmldialog(GTK_WINDOW(app_ui_data->app), NULL);

    if(importopmldialog != NULL)
        gtk_widget_show(importopmldialog);

}
void on_menu_properties(GtkMenuItem * menuitem, gpointer user_data)
{
    nodePtr ptr = ui_feedlist_get_selected();
    
    ULOG_DEBUG("on_menu_properties: CALLED \n\n\n\n\n\n\n\n\n\n\n\n");

    if (SFM_SEARCH == app_data->app_ui_data->search_mode){
        gtk_infoprint(GTK_WINDOW(app_data->app_ui_data->app), _("rss_ib_unable_edit_searching"));
        return;
    }

    if ((ptr != NULL) && (FST_FEED == ptr->type)) {
        on_popup_prop_selected((gpointer) ptr, 0, NULL);
    } else {
    // Should be on_feed_insensitive, but here for safety.
    gtk_infoprint(GTK_WINDOW(app_data->app_ui_data->app), _("rss_ib_no_feed_selected"));
    }
}

gboolean on_close(GtkWidget * widget, GdkEvent * event, gpointer user_data)
{
    //TODO: Remove SHOW_TRAY_ICON
    ULOG_DEBUG("ON_CLOSE");
    /*
    if (getBooleanConfValue(SHOW_TRAY_ICON) == FALSE)
    return on_quit(TRUE);
    gtk_widget_hide(mainwindow);
    return TRUE;
    */

    on_quit(TRUE);
    ULOG_DEBUG("******************************on_close: calling exit********************************");
//    exit(0); //just exit!!!!
//    ULOG_DEBUG("shouldn't happen");
}

void ui_help_cb()
{
    //g_assert(app_data != NULL);
    g_return_if_fail(app_data != NULL);

    //g_assert(app_data->osso);
    g_return_if_fail(app_data->osso);

    osso_return_t rc;

    rc = ossohelp_show(app_data->osso,
               RSS_NEWS_READER_HELP_TOPIC, 0);
    if (rc != OSSO_OK) {
    ULOG_ERR("Unable to show help: '%s' (errcode %d)",
         RSS_NEWS_READER_HELP_TOPIC, (int) rc);
    return;
    }
}

gboolean on_mainwindow_window_state_event(GtkWidget * widget,
                      GdkEvent * event,
                      gpointer user_data)
{

    if (!GTK_WIDGET_VISIBLE(mainwindow))
    return FALSE;

    if ((event->type) == (GDK_WINDOW_STATE)) {
    if ((((GdkEventWindowState *) event)->
         new_window_state) & GDK_WINDOW_STATE_MAXIMIZED)
        setBooleanConfValue(LAST_WINDOW_MAXIMIZED, TRUE);
    else
        setBooleanConfValue(LAST_WINDOW_MAXIMIZED, FALSE);
    }
    return FALSE;
}

struct file_chooser_tuple {
    GtkWidget *dialog;
    fileChoosenCallback func;
    gpointer user_data;
};

HildonApp *ui_create_main_app(void)
{
    HildonApp *result = NULL;
    ULOG_DEBUG("ui_create_main_app");
    /* Create HildonApp */
    result = HILDON_APP(hildon_app_new());

    /* Set title */
    hildon_app_set_title(result, _("rss_ti_newsreader"));

    return result;
}



/* Create the toolbar */
void ui_create_toolbar(AppUIData * data)
{
    /* Toolbar buttons */
    GtkToolItem *new_feed_button,
    *settings_button, *manage_folders_button, *remove_feed_button,
    *feed_properties_button,     *refresh_button, *search_button, *toolitem;
    GtkToolItem *separator;

    GtkToggleToolButton *folder_list_toggle_button;
    ULOG_DEBUG("ui_create_toolbar:");
    /* Paranoia */
    //g_assert(app_ui_data);
    g_return_if_fail(app_data);
    g_return_if_fail(app_data->app_ui_data);
    AppUIData * app_ui_data = app_data->app_ui_data;
    
    //g_assert(!app_ui_data->main_toolbar);
    g_return_if_fail(!app_ui_data->main_toolbar);

    app_ui_data->main_toolbar = gtk_toolbar_new();

    gtk_widget_show(app_ui_data->main_toolbar);
    
    gtk_toolbar_set_orientation(GTK_TOOLBAR(app_ui_data->main_toolbar),
                GTK_ORIENTATION_HORIZONTAL);
    gtk_container_set_border_width(GTK_CONTAINER
                   (app_ui_data->main_toolbar), 0);

    gtk_object_ref(GTK_OBJECT(app_ui_data->main_toolbar));
    gtk_object_sink(GTK_OBJECT(app_ui_data->main_toolbar));

    /*Refresh button*/
    ADD_TOOLBAR_BUTTON(refresh_button, "qgn_toolb_gene_refresh",
               on_refreshbtn_clicked)
    g_signal_connect(G_OBJECT(refresh_button), "insensitive_press",
             G_CALLBACK(on_refreshbtn_clicked), app_ui_data);
    app_ui_data->refresh_button = GTK_TOOL_BUTTON(refresh_button);
    app_ui_data->button_image_refresh =
    gtk_tool_button_get_icon_widget(app_ui_data->refresh_button);
    g_object_ref(app_ui_data->button_image_refresh);

    /*Add feed button*/
    ADD_TOOLBAR_BUTTON(new_feed_button, "qgn_toolb_rss_addfeed",
               on_newbtn_clicked_cb)
    g_signal_connect(G_OBJECT(new_feed_button), "insensitive_press",
             G_CALLBACK(on_newbtn_clicked_cb), NULL);

    app_ui_data->new_feed_button = GTK_TOOL_BUTTON(new_feed_button);               
    /* Settings button */
    ADD_TOOLBAR_BUTTON(settings_button, "qgn_toolb_rss_settings",
               on_prefbtn_clicked)
    /* Manage_folders button*/
    ADD_TOOLBAR_BUTTON(manage_folders_button,
               "qgn_toolb_rss_managefld",
               on_managefoldersbtn_clicked)
    g_signal_connect(G_OBJECT(manage_folders_button), "insensitive_press",
             G_CALLBACK(on_managefoldersbtn_clicked), NULL);

    app_ui_data->manage_folders_button = GTK_TOOL_BUTTON(manage_folders_button);

    /*folder_list_toggle_button. Cannot use the ADD_TOOL_BAR macro.
        Gotta use the GTK_TOGGLE_TOOL_BUTTON.
        If many toggle buttons needed later, we can make a macro
        ADD_TOGGLE_TOOL_BAR or something like that.
    */

    ADD_TOOLBAR_SEPARATOR

    folder_list_toggle_button = GTK_TOGGLE_TOOL_BUTTON (gtk_toggle_tool_button_new());

    gtk_tool_button_set_icon_widget (GTK_TOOL_BUTTON(folder_list_toggle_button),
      GTK_WIDGET (gtk_image_new_from_icon_name(  "qgn_toolb_rss_fldonoff",HILDON_ICON_SIZE_TOOLBAR) ));

    gtk_widget_show_all(GTK_WIDGET(folder_list_toggle_button));
    gtk_toggle_tool_button_set_active (folder_list_toggle_button, TRUE);

    GTK_WIDGET_UNSET_FLAGS(GTK_WIDGET(folder_list_toggle_button), GTK_CAN_FOCUS);
    g_signal_connect(G_OBJECT(folder_list_toggle_button), "clicked", 
            G_CALLBACK(on_folder_list_button_clicked), app_ui_data);
    gtk_toolbar_insert(GTK_TOOLBAR(app_ui_data->main_toolbar), 
            GTK_TOOL_ITEM(folder_list_toggle_button), -1);
    app_ui_data->folders_list_toggle_button =
        GTK_TOGGLE_TOOL_BUTTON(folder_list_toggle_button);

    /*
    ADD_TOOLBAR_SEPARATOR ADD_TOOLBAR_BUTTON(folder_list_toggle_button,
                         "qgn_toolb_rss_fldonoff",
                         on_folder_list_button_clicked)
    */

    /* Delete feed button */
    ADD_TOOLBAR_SEPARATOR ADD_TOOLBAR_BUTTON(remove_feed_button,
                         "qgn_toolb_rss_unsubscribefeed",
                         on_unsubscribe_clicked)
    g_signal_connect(G_OBJECT(remove_feed_button), "insensitive_press",
             G_CALLBACK(on_unsubscribe_clicked), NULL);

    app_ui_data->unsubscribe_button =
    GTK_TOOL_BUTTON(remove_feed_button);
    /* feed properties button */
    ADD_TOOLBAR_BUTTON(feed_properties_button, "qgn_toolb_rss_feedprop",
               on_menu_properties)

    g_signal_connect(G_OBJECT(feed_properties_button), "insensitive_press",
             G_CALLBACK(on_menu_properties), NULL);

    app_ui_data->feed_properties_button =
    GTK_TOOL_BUTTON(feed_properties_button);

    ADD_TOOLBAR_SEPARATOR
    app_ui_data->var_field = gtk_alignment_new(0, 0.5, 1, 0);
    gtk_widget_show(app_ui_data->var_field);

    toolitem = gtk_tool_item_new();
    gtk_widget_show(GTK_WIDGET(toolitem));
    gtk_tool_item_set_expand(GTK_TOOL_ITEM(toolitem), TRUE);
    gtk_container_add(GTK_CONTAINER(toolitem), app_ui_data->var_field);
    
    app_ui_data->search_field = gtk_combo_box_entry_new_text();
    gtk_widget_show(app_ui_data->search_field);
    g_object_ref(app_ui_data->search_field);


    app_ui_data->progress_bar = gtk_progress_bar_new();
    gtk_widget_show(app_ui_data->progress_bar);
    g_object_ref(app_ui_data->progress_bar);
    gtk_container_add(GTK_CONTAINER(app_ui_data->var_field),
              GTK_WIDGET(app_ui_data->search_field));
    gtk_toolbar_insert(GTK_TOOLBAR(app_ui_data->main_toolbar),
               GTK_TOOL_ITEM(toolitem), -1);

    ADD_TOOLBAR_BUTTON(search_button, "qgn_toolb_browser_gobutton",
               on_searchentry_activate)
    app_ui_data->search_button = GTK_TOOL_BUTTON(search_button);
    app_ui_data->button_image_search =
    gtk_tool_button_get_icon_widget(app_ui_data->search_button);
    g_object_ref(app_ui_data->button_image_search);

    /* Set toolbar properties */
    gtk_toolbar_set_style(GTK_TOOLBAR(app_ui_data->main_toolbar),
              GTK_TOOLBAR_ICONS);

    hildon_appview_set_toolbar(hildon_app_get_appview(app_ui_data->app),
                   GTK_TOOLBAR(app_ui_data->main_toolbar));

    app_ui_data->button_image_stop =
    gtk_image_new_from_icon_name("qgn_toolb_gene_stop",
                     GTK_ICON_SIZE_LARGE_TOOLBAR);
    gtk_widget_show(app_ui_data->button_image_stop);
    g_object_ref(app_ui_data->button_image_stop);

    return;
}


/* Create the main window */
void ui_create_main_window(AppData * app_data)
{
    AppUIData *app_ui_data;

    /* Paranoia */
    //g_assert(app_data);
    //g_assert(app_data->app_ui_data);
    
    app_ui_data = app_data->app_ui_data;

    app_ui_data->app = ui_create_main_app();

    gtk_widget_add_events(GTK_WIDGET(app_ui_data->app),
              GDK_KEY_RELEASE_MASK | GDK_KEY_PRESS_MASK);
    
    g_signal_connect((gpointer) app_ui_data->app, "key-release-event",
             G_CALLBACK(key_release), app_ui_data);

    g_signal_connect((gpointer) app_ui_data->app, "key-press-event",
             G_CALLBACK(key_press), app_ui_data);
    
    g_signal_connect(G_OBJECT(app_ui_data->app),
             "topmost_status_lose",
             G_CALLBACK(on_untop), NULL);

    g_signal_connect(G_OBJECT(app_ui_data->app),
             "topmost_status_acquire",
             G_CALLBACK(on_top), app_data);

    /* This callback is needed only for debugging. */
/*  gtk_signal_connect(GTK_OBJECT(app_ui_data->app),
                     "key_press_event",
                     G_CALLBACK(on_key_press),
                     app_data); */
    
    app_ui_data->main_view = HILDON_APPVIEW(hildon_appview_new(NULL));
    gtk_widget_show_all(GTK_WIDGET(app_ui_data->main_view));    /* show_all to make toolbar visible */

    /* Connect destroy signal */
    g_signal_connect(G_OBJECT(app_ui_data->main_view), "destroy",
             G_CALLBACK(gtk_main_quit), NULL);

    hildon_app_set_appview(app_ui_data->app, app_ui_data->main_view);

    ui_create_toolbar(app_ui_data);

    return;
}

void ui_set_folders_menu_item_state(AppUIData * data,
                    gboolean state)
{
    //g_assert(app_ui_data != NULL);
    AppUIData *app_ui_data;
    app_ui_data = app_data->app_ui_data;
    
    g_signal_handlers_block_by_func(G_OBJECT
                    (app_ui_data->menuitem_folders),
                    G_CALLBACK(toggle_folder_list), NULL);
    gtk_check_menu_item_set_active(app_ui_data->menuitem_folders, state);
    g_signal_handlers_unblock_by_func(G_OBJECT
                      (app_ui_data->menuitem_folders),
                      G_CALLBACK(toggle_folder_list),
                      NULL);
}

void ui_set_fullscreen_menu_item_state(AppUIData * data,
                       gboolean state)
{
    //g_assert(app_ui_data != NULL);
    AppUIData *app_ui_data;
    app_ui_data = app_data->app_ui_data;
    

    g_signal_handlers_block_by_func(G_OBJECT
                    (app_ui_data->menuitem_fullscreen),
                    G_CALLBACK(on_toggle_fullscreen_mode),
                    NULL);
    gtk_check_menu_item_set_active(app_ui_data->menuitem_fullscreen,
                   state);
    g_signal_handlers_unblock_by_func(G_OBJECT
                      (app_ui_data->menuitem_fullscreen),
                      G_CALLBACK
                      (on_toggle_fullscreen_mode), NULL);
}

void ui_set_toolbar_full_menu_item_state(AppUIData * data,
                     gboolean state)
{
    //g_assert(app_ui_data != NULL);
    AppUIData *app_ui_data;
    app_ui_data = app_data->app_ui_data;

    g_signal_handlers_block_by_func(G_OBJECT(app_ui_data->menuitem_full),
                    G_CALLBACK(toggle_toolbar), NULL);
    gtk_check_menu_item_set_active(app_ui_data->menuitem_full, state);
    g_signal_handlers_unblock_by_func(G_OBJECT(app_ui_data->menuitem_full),
                      G_CALLBACK(toggle_toolbar), NULL);
}

void ui_set_toolbar_normal_menu_item_state(AppUIData * app_ui_data,
                       gboolean state)
{
    //g_assert(app_ui_data != NULL);
    
    
    g_signal_handlers_block_by_func(G_OBJECT(app_ui_data->menuitem_normal),
                    G_CALLBACK(toggle_toolbar), NULL);
    gtk_check_menu_item_set_active(app_ui_data->menuitem_normal, state);
    g_signal_handlers_unblock_by_func(G_OBJECT
                      (app_ui_data->menuitem_normal),
                      G_CALLBACK(toggle_toolbar), NULL);
}


void ui_set_folder_list_state(AppUIData * app_ui_data,
                  gboolean state, gboolean update_menu_item)
{
    g_assert(app_data != NULL);
    g_assert(app_data->app_ui_data != NULL);

    if (state) {
    gtk_widget_show(feedlist_container);
    } else {
        gtk_widget_hide(feedlist_container);
    }
    if (update_menu_item) {
    ui_set_folders_menu_item_state(app_ui_data, state);
    }
}


void ui_set_fullscreen_state(AppUIData * app_ui_data,
                 gboolean state, gboolean update_menu_item)
{
    GtkWidget *toolbar;
    gboolean show_toolbar = FALSE;

    g_assert(app_ui_data != NULL);
    
    toolbar = app_ui_data->main_toolbar;
    
    if (state) {
    hildon_appview_set_fullscreen(hildon_app_get_appview
                      (app_ui_data->app), TRUE);
    show_toolbar = gtk_check_menu_item_get_active
        (app_ui_data->menuitem_full);
    
    ULOG_INFO("Application now in fullscreen mode");
    } else {
    hildon_appview_set_fullscreen(hildon_app_get_appview
                      (app_ui_data->app), FALSE);
    show_toolbar = gtk_check_menu_item_get_active
        (app_ui_data->menuitem_normal);
    ULOG_INFO("Application now out of fullscreen mode.");
    }
    if (show_toolbar) {
    gtk_widget_show(toolbar);
    }
    // should always show toolbar if not in normal mode as user needs to see the
    // progressbar
    else if (app_ui_data->search_mode == SFM_NORMAL){
    gtk_widget_hide(toolbar);
    }
    if (update_menu_item) {
    ui_set_fullscreen_menu_item_state(app_ui_data, state);
    }
}


ConfirmationResponse
ui_show_folder_delete_confirmation(AppUIData * app_ui_data, gint n_feeds)
{
    HildonNote *note = NULL;
    gint resp1 = CONFRESP_ERROR;
    ConfirmationResponse resp = CONFRESP_ERROR;
    gchar *str = NULL, *str2 = NULL;

    str = g_strdup_printf("%s", _("rss_nc_folder_deletion_will_affect_feeds"));
    str2 = g_strdup_printf(str, n_feeds);
    
    note = HILDON_NOTE
    (hildon_note_new_confirmation_add_buttons
     (NULL,
      str2,
      _("rss_bd_ok"),
      CONFRESP_OK, _("rss_bd_cancel"), CONFRESP_CANCEL, NULL, NULL));
    if (note == NULL) {
    ULOG_CRIT("Unable to create HildonNote");
    g_free(str);
    g_free(str2);
    str = NULL;
    str2 = NULL;
    return CONFRESP_ERROR;
    }

    /* show dialog */
    resp1 = gtk_dialog_run(GTK_DIALOG(note));
    gtk_widget_destroy(GTK_WIDGET(note));
    g_free(str);
    g_free(str2);
    str = NULL;
    str2 = NULL;
    resp = (resp1 == CONFRESP_OK || resp1 == CONFRESP_CANCEL)
    ? resp1 : CONFRESP_ERROR;
    return resp;
}

ConfirmationResponse
ui_show_feed_delete_confirmation(AppUIData * app_ui_data,
                 gchar * feed_title)
{
    HildonNote *note = NULL;
    gint resp1 = CONFRESP_ERROR;
    ConfirmationResponse resp = CONFRESP_ERROR;
    gchar *str = NULL, *str2 = NULL;

    str = g_strdup_printf("%s", _("rss_nc_are_you_sure_unsubscribe"));
    str2 = g_strdup_printf(str, feed_title ? feed_title : "");
    
    note = HILDON_NOTE
    (hildon_note_new_confirmation_add_buttons
     (NULL,
      str2,
      _("rss_bd_ok"),
      CONFRESP_OK, _("rss_bd_cancel"), CONFRESP_CANCEL, NULL, NULL));
    if (note == NULL) {
    ULOG_CRIT("Unable to create HildonNote");
    g_free(str);
    g_free(str2);
    str = NULL;
    str2 = NULL;
    return CONFRESP_ERROR;
    }

    /* show dialog */
    resp1 = gtk_dialog_run(GTK_DIALOG(note));
    gtk_widget_destroy(GTK_WIDGET(note));
    g_free(str);
    g_free(str2);
    str = NULL;
    str2 = NULL;
    resp = (resp1 == CONFRESP_OK || resp1 == CONFRESP_CANCEL)
    ? resp1 : CONFRESP_ERROR;
    return resp;
}

void ui_mainwindow_show_statistics()
{
    ULOG_DEBUG("ui_mainwindow_show_statistics...........");
    ui_htmlview_clear(ui_mainwindow_get_active_htmlview());
    ui_itemlist_clear();
    ui_mainwindow_finish();
    
    g_assert(app_data != NULL);
    g_assert(app_data->app_ui_data != NULL);
    
    hildon_appview_set_title(app_data->app_ui_data->main_view, "");
}

void ui_mainwindow_save_search_entries()
{
    gchar *filename = NULL;
    gchar *entry_text = NULL;
    FILE *file = NULL;
    GtkComboBox *search_combo;
    int i = 0;
    int search_string_count = 0;
    
    g_assert(app_data != NULL);
    g_assert(app_data->app_ui_data != NULL);
    
    ULOG_DEBUG("**********ui_mainwindow_save_search_entries: saving search entries ");

    filename =
    g_strdup_printf("%s" G_DIR_SEPARATOR_S ".osso_rss_feed_reader"
            G_DIR_SEPARATOR_S "searchentries", g_get_home_dir());
    
    if (g_file_test(filename, G_FILE_TEST_EXISTS))
    g_unlink(filename);
    
    file = fopen(filename, "wb");
    
    if(file == NULL)
    {
        ULOG_DEBUG("Could not save the search entries");
    g_free(filename);
    return;
    }
    
    search_combo = GTK_COMBO_BOX(app_data->app_ui_data->search_field);

    if(search_combo == NULL)
    {
        g_free(filename);
        return;
    }
  
    search_string_count = app_data->app_ui_data->search_string_count;
        
    cache_write_int(file, search_string_count);
    
    for(i = 0; i < search_string_count; i++)
    {
        gtk_combo_box_set_active(search_combo, i);
        entry_text = gtk_combo_box_get_active_text(search_combo);
       cache_write_str(file, entry_text);
       ULOG_DEBUG("Wrote: %s", entry_text);
       g_free(entry_text);
    }
    fclose(file);
    g_free(filename);
}

void ui_mainwindow_load_search_entries()
{
    gchar *filename = NULL;
    FILE *file = NULL;
    GtkComboBox *search_combo;
    int i = 0;
    int search_string_count = 0;
    gchar *tmp;
    
    g_assert(app_data != NULL);
    g_assert(app_data->app_ui_data != NULL);
    
    ULOG_DEBUG("ui_mainwindow_load_search_entries");
    
    filename =
    g_strdup_printf("%s" G_DIR_SEPARATOR_S ".osso_rss_feed_reader"
            G_DIR_SEPARATOR_S "searchentries", g_get_home_dir());
    
    if (!g_file_test(filename, G_FILE_TEST_EXISTS))
    {
        ULOG_DEBUG("Search entry file not found");
    g_free(filename);
    return;
    }
    
    file = fopen(filename, "rb");
    
    if(file == NULL)
    {
        ULOG_DEBUG("Could not load the search entries");
    g_free(filename);
    return;
    }
    
    search_combo = GTK_COMBO_BOX(app_data->app_ui_data->search_field);

    if(search_combo == NULL)
    {
        g_free(filename);
        return;
    }
    
    search_string_count = cache_read_int(file);
    app_data->app_ui_data->search_string_count = search_string_count;
    
    ULOG_DEBUG("ui_mainwindow_load_search_entries: search_string_count = %d", search_string_count);
    for(i = 0; i < search_string_count; i++){
        tmp = cache_read_str(file);
        gtk_combo_box_append_text(search_combo, tmp);
        g_free(tmp);
    }
    fclose(file);
    g_free(filename);
}

void ui_mainwindow_empty_search_field()
{
    GtkWidget *search_entry;
    
    g_assert(app_data != NULL);
    g_assert(app_data->app_ui_data != NULL);
    
    search_entry = gtk_bin_get_child(GTK_BIN(app_data->app_ui_data->search_field));
    
    g_assert(NULL != search_entry);
    
    gtk_entry_set_text(GTK_ENTRY(search_entry), "");
}
